# coding:utf-8
import sys
from tools_hjh import DBConn
from tools_hjh import Log
from tools_hjh import ThreadPool
from tools_hjh import Tools
from tools_hjh import HTTPTools
import json
import time
import datetime
from dateutil.relativedelta import relativedelta
import math
from tools_hjh.Tools import str_exception
from bs4 import BeautifulSoup

stop_flag = False
page_num = None

date = Tools.locatdate()
log = Log('E:/MyFiles/MyPy/log/fund/' + date + '.log')

host = 'https://'

chrome_path = r'D:\MyApps\CentBrowser\App\chrome.exe'
chromedriver_path = r'D:\MyApps\CentBrowser\chromedriver.exe'

headers = {
    'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
    'accept-encoding': 'text',
    'accept-language': 'zh-CN,zh;q=0.9',
    'cache-control': 'no-cache',
    'dnt': '1',
    'pragma': 'no-cache',
    'sec-ch-ua': 'x86;v=99, Windows;v=10, Surface Laptop Studio;v=1',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-platform': '"Windows"',
    'Referer':'http://fundf10.eastmoney.com/',
    'user-agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.101 Safari/537.36',
    'cookie':'qgqp_b_id=83a199356104c7aa279534a8ad26f63a; emshistory=%5B%22%E7%BE%8E%E5%85%83%E5%85%91%E7%A6%BB%E5%B2%B8%E4%BA%BA%E6%B0%91%E5%B8%81%22%5D; st_nvi=CcZVuMTz5IhmsufyNCKse50b5; nid=00e15026f1062a9c810d5874360ccee6; nid_create_time=1753455327603; gvi=VkSGuR16GXiIJZI_ZREui577f; gvi_create_time=1753455327603; nid_id=148913046; websitepoptg_api_time=1754570340785; ASP.NET_SessionId=b3ymoksfubguqfwo5c14g0j1; st_si=75005746448136; st_asi=delete; searchbar_code=007229_007228_008566_006210_006212_009018_001247_019533_486002_510080; EMFUND0=08-07%2022%3A06%3A59@%23%24%u534E%u5B89%u4E2D%u503A7-10%u5E74%u56FD%u5F00%u503AC@%23%24007229; EMFUND1=08-07%2022%3A07%3A21@%23%24%u534E%u5B89%u4E2D%u503A7-10%u5E74%u56FD%u5F00%u503AA@%23%24007228; EMFUND2=08-07%2022%3A08%3A35@%23%24%u8702%u5DE2%u6DFB%u76C8%u7EAF%u503AA@%23%24008566; EMFUND3=08-07%2022%3A25%3A37@%23%24%u4E1C%u65B9%u81FB%u5B9D%u7EAF%u503A%u503A%u5238A@%23%24006210; EMFUND4=08-07%2022%3A33%3A33@%23%24%u897F%u90E8%u5229%u5F97%u805A%u6CF018%u4E2A%u6708%u5B9A%u5F00%u503AA@%23%24009018; EMFUND5=08-08%2000%3A34%3A43@%23%24%u534E%u6CF0%u67CF%u745E%u65B0%u5229%u6DF7%u5408A@%23%24001247; EMFUND6=08-08%2012%3A04%3A47@%23%24%u946B%u5143%u6CFD%u5229C@%23%24019533; EMFUND9=08-08%2022%3A12%3A17@%23%24%u957F%u76DB%u5168%u503A%u6307%u6570%u589E%u5F3A%u503A%u5238A@%23%24510080; fullscreengg=1; fullscreengg2=1; EMFUND8=08-09%2001%3A08%3A39@%23%24%u5DE5%u94F6%u5168%u7403%u7CBE%u9009%u80A1%u7968%28QDII%29@%23%24486002; EMFUND7=08-09 01:12:00@#$%u5DE5%u94F6%u5168%u7403%u80A1%u7968%28QDII%29%u4EBA%u6C11%u5E01@%23%24486001; st_pvi=49894389850834; st_sp=2025-07-17%2004%3A08%3A29; st_inirUrl=https%3A%2F%2Fquote.eastmoney.com%2Fgb%2FzsNDX100.html; st_sn=49; st_psi=2025080901120046-112200305282-5761806720'
}

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = 'download'
try:
    sys_argv_2 = int(sys.argv[2])
except:
    sys_argv_2 = -1  # 获取条数
try:
    sys_argv_3 = sys.argv[3]
except:
    sys_argv_3 = 2
    
thread_num = 64


def main(): 
    if sys_argv_3 == 1:
        db = DBConn('pgsql', host='127.0.0.1', port=5432, db='fund', username='postgres', password='postgres', poolsize=8)
        try:
            createDatabase(db, rebuild=False)
            get_fund_main(db, HTTPTools)
            get_ndx(db)
            get_ndx_pe(db)
            # get_fund_dv(db, HTTPTools, begin_code='000000', code=None, name_like='%国泰纳斯达克100指数%')
            # get_fund_dv(db, HTTPTools, begin_code='000000', code=None, name_like='%纳斯达克%')
            # get_fund_dv(db, HTTPTools, begin_code='000000', code=None, name_like='%纳指%')
            get_fund_dv(db, HTTPTools, begin_code='000000', code=None, name_like='%%')
            get_zf(db)
            get_val(db)
            get_hc(db)
            get_no_new_high_days(db)
            get_ret_ann(db)
            # get_excel()
            # backtesting(db)
        finally:
            db.close()
            pass
    else:
        get_excel()


def createDatabase(db, rebuild=False):
    if rebuild:
        db.run('drop table if exists t_m')
        db.run('drop table if exists t_e')
        db.run('drop table if exists t_e_imf')

    t_m = '''
        create table if not exists t_m(
            code varchar(9),
            name varchar(255),
            type varchar(255),
            primary key(code)
        )
    '''
    
    t_e = '''
        create table if not exists t_e(
            code varchar(9),
            date date,
            dwjz real,
            ljjz real,
            zf real,
            fh varchar(255),
            val real,
            hc real,
            ret_1y_ann real,
            ret_3y_ann real,
            ret_5y_ann real,
            ret_6m real,
            ret_3m real,
            ret_1m real,
            primary key(code, date)
        )
    '''
    
    # 货币基金的每日滚动7日年化
    t_e_imf = '''
        create table if not exists t_e_imf(
            code varchar(9),
            date date,
            ret_7d_ann real,
            primary key(code, date)
        )
    '''
        
    idx_e_code_date_ret_1y_ann = '''
        CREATE INDEX IF NOT EXISTS idx_e_code_date_syl
        ON t_e (code, date)
        INCLUDE (ret_1y_ann, ret_3y_ann, ret_5y_ann, hc, zf)
    '''
    
    tmp_code_mindate_1 = '''
        insert into tmp_code_mindate select code,min(date) from t_e t where t.code not in (select code from tmp_code_mindate) group by code
    '''
    tmp_code_mindate_2 = '''
        insert into tmp_code_mindate select code,min(date) from t_e_imf t where t.code not in (select code from tmp_code_mindate) group by code
    '''
    
    db.run(t_m)
    db.run(t_e)
    db.run(t_e_imf)
    db.run(idx_e_code_date_ret_1y_ann)
    db.run(tmp_code_mindate_1)
    db.run(tmp_code_mindate_2)


def get_fund_main(db, rep):
    sql = 'insert into t_m(code,name,type) values(?,?,?) on conflict(code) do update set name = EXCLUDED.name, type = EXCLUDED.type'
    page = rep.get('http://fund.eastmoney.com/js/fundcode_search.js')
    page = page.encode('utf8')[3:].decode('utf8')
    page = page.replace('var r = ', '').replace(';', '')
    fund_list = json.loads(page)
    params = []
    for fund in fund_list:
        code = fund[0]
        name = fund[2]
        type_ = fund[3]
        params.append((code, name, type_))
    num = db.run(sql, params)
    db.run("update t_m set type = '货币型-浮动净值' where code in('511880','003816')")
    log.info('get_fund_main', num)

    
def get_fund_top10_holdings(db, rep):
    pass


# 从库中最大的date开始，获取至今的数据，如果中间缺了不会补
def get_fund_dv(db, rep, begin_code='000000', code=None, name_like=''):
            
    sql = "insert into t_e(code,date,dwjz,ljjz,fh) values(?,?,?,?,?) on conflict(code,date) do update set dwjz = EXCLUDED.dwjz, ljjz = EXCLUDED.ljjz, fh = EXCLUDED.fh"
    sql_imf = "insert into t_e_imf(code,date,ret_7d_ann) values(?,?,?) on conflict(code,date) do update set ret_7d_ann = EXCLUDED.ret_7d_ann"
    vals_all = []
    imfs_all = []

    def get_fund_dv_one(code, type_, rep, times=1):
        try:
            page_size = 20
            timestamp = str(int(time.time() * 1000))
            end_date = Tools.locatdate()
            start_date = db.run("select to_char(max(date),'YYYY-MM-DD') from t_e where code = ?", (code,)).get_rows()[0][0]
            if start_date is None:
                start_date = '1949-10-01'
            if end_date == start_date:
                log.info('get_fund_dv', code, 0, len(vals_all), times)
                return
            
            all_size = (datetime.datetime.strptime(end_date, "%Y-%m-%d").date() - datetime.datetime.strptime(start_date, "%Y-%m-%d").date()).days
            # all_size = 230
            page_num = math.ceil(all_size / page_size)
            
            rss_num = 0
            for page_idx in range(1, page_num + 1):
                url = 'http://api.fund.eastmoney.com/f10/lsjz?callback=jQuery18306743973867400965_1722217638986&fundCode=' + code + '&pageIndex=' + str(page_idx) + '&pageSize=' + str(page_size) + '&startDate=' + start_date + '&endDate=' + end_date + '&_=' + timestamp
                page = rep.get(url, headers=headers)
                # Tools.echo(page, 'E:\page.txt')
                page = page.replace('jQuery18306743973867400965_1722217638986', '').strip()
                page = page.split('"LSJZList":')[1].split(',"FundType"')[0]
                rss = json.loads(page)
                rss_num = rss_num + len(rss)
                
                if len(rss) == 0:
                    break
                
                for rs in reversed(rss):
                    date_ = rs['FSRQ']
                    dwjz = rs['DWJZ']
                    if dwjz == '':
                        dwjz = None
                    ljjz = rs['LJJZ']
                    if ljjz == '':
                        ljjz = None
                    zf = rs['JZZZL']
                    if zf == '':
                        zf = 0
                    fh = rs['FHSP']
                    if fh == '':
                        fh = None
         
                    if type_ == '货币型-普通货币':
                        imfs_all.append((code, date_, ljjz))
                    else:
                        vals_all.append((code, date_, dwjz, ljjz, fh))

            log.info('get_fund_dv', code, rss_num, len(vals_all) + len(imfs_all), times)
                    
        except Exception as _:
            if times <= 99:
                log.warning('get_fund_dv', code, times, str_exception(_))
                time.sleep(1)
                times = times + 1
                get_fund_dv_one(code, rep, times)
            else:
                log.error('get_fund_dv', code, times, str_exception(_))
    
    if code is not None:
        get_code_sql = '''
            select code,type from t_m 
            where code = ?
        '''
        funds = db.run(get_code_sql, (code,)).get_rows()
    else:
        get_code_sql = '''
            select code,type from t_m 
            where 1=1
            -- and type not in('货币型-普通货币')
            and code >= ?
            and name like ?
            order by code
        '''
        funds = db.run(get_code_sql, (begin_code, name_like)).get_rows()
    
    tp = ThreadPool(thread_num)
    for fund in funds:
        if len(vals_all) >= 1000000 or len(imfs_all) >= 1000000:
            tp.wait()
            if len(vals_all) >= 1000000:
                num = db.run(sql, vals_all)
                log.info('get_fund_dv', num)
                vals_all.clear()
            if len(imfs_all) >= 1000000:
                num = db.run(sql_imf, imfs_all)
                log.info('get_fund_dv', num)
                imfs_all.clear()
        else:
            code = fund[0]
            type_ = fund[1]
            tp.run_wait(get_fund_dv_one, (code, type_, rep))
            
    tp.wait()
    num1 = db.run(sql, vals_all)
    num2 = db.run(sql_imf, imfs_all)
    num = num1 + num2
    log.info('get_fund_dv', num)
    vals_all.clear()


def get_zf(db):

    def get_zf_one(db, code):
        sql = 'update t_e set zf = ? where code = ? and date = ?'
        params = []
        rss = db.run('select date,dwjz,fh from t_e where code = ? and zf is null order by date', (code,)).get_rows()
        for rs in rss:
            date_ = rs[0]
            dwjz = float(rs[1])
            fh = rs[2]
            try:
                previous_dwjz = float(db.run('select dwjz from t_e where code = ? and date < ? order by date desc limit 1', (code, date_)).get_rows()[0][0])
            except:
                zf = 0
                params.append((zf, code, date_))
                continue
            
            if '每份基金份额分拆' in str(fh):
                fh_cf = float(fh.replace('每份基金份额分拆', '').replace('份', ''))
                zf = ((dwjz * fh_cf) - previous_dwjz) / previous_dwjz * 100
            elif '每份派现金' in str(fh):
                fh_xj = float(fh.replace('每份派现金', '').replace('元', ''))
                zf = ((dwjz + fh_xj) - previous_dwjz) / previous_dwjz * 100
            elif '每份基金份额折算' in str(fh):
                fh_zs = float(fh.replace('每份基金份额折算', '').replace('份', ''))
                zf = (dwjz - (previous_dwjz / fh_zs)) / (previous_dwjz / fh_zs) * 100
            else:
                zf = (dwjz - previous_dwjz) / previous_dwjz * 100
            
            params.append((zf, code, date_))
        try:
            num = db.run(sql, params)
            if num > 0:
                log.info('get_zf', code, num, len(params))
        except Exception as _:
            log.error('get_zf', code, str(_))

    tp = ThreadPool(64)
    log.info('get_zf', 'begin')
    funds = db.run("select code from t_m order by code").get_rows()
    
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and dwjz is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_zf_one, (db, fund[0]))
    tp.wait()

    
def get_val(db):

    def get_val_one(db, code):
        sql = 'update t_e set val = ? where code = ? and date = ?'
        params = []   
        rss = db.run('select date,zf from t_e where code = ? and val is null order by date', (code,)).get_rows()
        next_val = 1 
        for rs in rss:
            try:
                date_ = rs[0]
                zf = rs[1]
                try:
                    next_val_db = None
                    next_val_db = db.run('select val from t_e where code = ? and date < ? order by date desc limit 1', (code, date_)).get_rows()[0][0]
                except:
                    next_val_db = None
                if next_val_db is not None:
                    next_val = next_val_db
                val = float(next_val) * (1 + float(zf) / 100)
                params.append((val, code, date_))
                next_val = val
            except Exception as _:
                print(_, code, date_, next_val, zf)
        try:
            num = db.run(sql, params)
            if num > 0:
                log.info('get_val', code, num, len(params))
        except Exception as _:
            log.error('get_val', code, str(_))

    tp = ThreadPool(64)
    log.info('get_val', 'begin')
    funds = db.run("select code from t_m order by code").get_rows()
    
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and zf is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_val_one, (db, fund[0]))
    tp.wait()


def get_hc(db):

    def get_hc_one(db, code):
        params = []      
        sql = 'update t_e set hc = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where hc is null and code = ? order by date', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_before = date_now - relativedelta(years=1)
            val = row[1]
            max_val = db.run('select max(val) from t_e where date <= ? and date >= ? and code = ?', (date_now, date_before, code)).get_rows()[0][0]
            val = float(val)
            max_val = float(max_val)
            hc = round((max_val - val) / max_val * 100, 3)
            params.append((hc, code, date_now))
        try:
            num = db.run(sql, params)
            if num > 0:
                log.info('get_hc', code, num, len(params))
        except Exception as _:
            log.error('get_hc', code, str(_))
    
    tp = ThreadPool(64)
    log.info('get_hc', 'begin')
    funds = db.run("select code from t_m order by code").get_rows()
    
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and val is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_hc_one, (db, fund[0]))
    tp.wait()

    
def get_no_new_high_days(db):

    def get_no_new_high_days_one(db, code):
        params = []      
        sql = 'update t_e set no_new_high_days = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where no_new_high_days is null and code = ? order by date', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            val = row[1]
            date_before, max_val = db.run('select date,val from t_e where date <= ? and code = ? order by val desc limit 1', (date_now, code)).get_rows()[0]
            if max_val <= val:
                no_new_high_days = 0
            else:
                no_new_high_days = (date_now - date_before).days
            params.append((no_new_high_days, code, date_now))
        try:
            num = db.run(sql, params)
            if num > 0:
                log.info('get_no_new_high_days', code, num, len(params))
        except Exception as _:
            log.error('get_no_new_high_days', code, str(_))
    
    tp = ThreadPool(64)
    log.info('get_no_new_high_days', 'begin')
    funds = db.run("select code from t_m order by code").get_rows()
    
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and val is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_no_new_high_days_one, (db, fund[0]))
    tp.wait()


def get_ret_ann(db):

    def get_ret_1y_ann_one(db, code):
        ret_1y_ann_params = []
        ret_1y_ann_sql = 'update t_e set ret_1y_ann = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where ret_1y_ann is null and code = ? order by date', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_before = date_now - relativedelta(years=1)
            val = row[1]
            val = float(val)
            try:
                val_before = db.run('select val from t_e where date >= ? and code = ? order by date limit 1', (date_before, code)).get_rows()[0][0]
                val_before = float(val_before)
                ret_1y_ann = round((val - val_before) / val_before * 100, 3)
            except Exception as _:
                ret_1y_ann = None
            if ret_1y_ann is None:
                break
            ret_1y_ann_params.append((ret_1y_ann, code, date_now))
        try:
            num = db.run(ret_1y_ann_sql, ret_1y_ann_params)
            if num > 0:
                log.info('get_ret_1y_ann_one', code, num, len(ret_1y_ann_params))
        except Exception as _:
            log.error('get_ret_1y_ann_one', code, str(_))
        
    def get_ret_3y_ann_one(db, code):
        ret_3y_ann_params = []
        ret_3y_ann_sql = 'update t_e set ret_3y_ann = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where ret_3y_ann is null and code = ? order by date', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_before = date_now - relativedelta(years=3)
            val = row[1]
            val = float(val)
            try:
                val_before = db.run('select val from t_e where date >= ? and code = ? order by date limit 1', (date_before, code)).get_rows()[0][0]
                val_before = float(val_before)
                ret_3y_ann = round((val - val_before) / val_before / 3 * 100, 3)
            except Exception as _:
                ret_3y_ann = None
            if ret_3y_ann is None:
                break
            ret_3y_ann_params.append((ret_3y_ann, code, date_now))
        try:
            num = db.run(ret_3y_ann_sql, ret_3y_ann_params)
            if num > 0:
                log.info('get_ret_3y_ann_one', code, num, len(ret_3y_ann_params))
        except Exception as _:
            log.error('get_ret_3y_ann_one', code, str(_))
        
    def get_ret_5y_ann_one(db, code):
        ret_5y_ann_params = []
        ret_5y_ann_sql = 'update t_e set ret_5y_ann = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where ret_5y_ann is null and code = ? order by date', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_before = date_now - relativedelta(years=5)
            val = row[1]
            val = float(val)
            try:
                val_before = db.run('select val from t_e where date >= ? and code = ? order by date limit 1', (date_before, code)).get_rows()[0][0]
                val_before = float(val_before)
                ret_5y_ann = round((val - val_before) / val_before / 5 * 100, 3)
            except:
                ret_5y_ann = None
            if ret_5y_ann is None:
                break
            ret_5y_ann_params.append((ret_5y_ann, code, date_now))
        try:
            num = db.run(ret_5y_ann_sql, ret_5y_ann_params)
            if num > 0:
                log.info('get_ret_5y_ann_one', code, num, len(ret_5y_ann_params))
        except Exception as _:
            log.error('get_ret_5y_ann_one', code, str(_))
            
    def get_ret_6m_one(db, code):
        ret_6m_params = []
        ret_6m_sql = 'update t_e set ret_6m = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where ret_6m is null and code = ? order by date', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_before = date_now - relativedelta(months=6)
            val = row[1]
            val = float(val)
            try:
                val_before = db.run('select val from t_e where date >= ? and code = ? order by date limit 1', (date_before, code)).get_rows()[0][0]
                val_before = float(val_before)
                ret_6m = round((val - val_before) / val_before * 100, 3)
            except:
                ret_6m = None
            if ret_6m is None:
                break
            ret_6m_params.append((ret_6m, code, date_now))
        try:
            num = db.run(ret_6m_sql, ret_6m_params)
            if num > 0:
                log.info('get_ret_6m_one', code, num, len(ret_6m_params))
        except Exception as _:
            log.error('get_ret_6m_one', code, str(_))

    def get_ret_3m_one(db, code):
        ret_3m_params = []
        ret_3m_sql = 'update t_e set ret_3m = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where ret_3m is null and code = ? order by date', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_before = date_now - relativedelta(months=3)
            val = row[1]
            val = float(val)
            try:
                val_before = db.run('select val from t_e where date >= ? and code = ? order by date limit 1', (date_before, code)).get_rows()[0][0]
                val_before = float(val_before)
                ret_3m = round((val - val_before) / val_before * 100, 3)
            except:
                ret_3m = None
            if ret_3m is None:
                break
            ret_3m_params.append((ret_3m, code, date_now))
        try:
            num = db.run(ret_3m_sql, ret_3m_params)
            if num > 0:
                log.info('get_ret_3m_one', code, num, len(ret_3m_params))
        except Exception as _:
            log.error('get_ret_3m_one', code, str(_))

    def get_ret_1m_one(db, code):
        ret_1m_params = []
        ret_1m_sql = 'update t_e set ret_1m = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where ret_1m is null and code = ? order by date', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_before = date_now - relativedelta(months=6)
            val = row[1]
            val = float(val)
            try:
                val_before = db.run('select val from t_e where date >= ? and code = ? order by date limit 1', (date_before, code)).get_rows()[0][0]
                val_before = float(val_before)
                ret_1m = round((val - val_before) / val_before * 100, 3)
            except:
                ret_1m = None
            if ret_1m is None:
                break
            ret_1m_params.append((ret_1m, code, date_now))
        try:
            num = db.run(ret_1m_sql, ret_1m_params)
            if num > 0:
                log.info('get_ret_1m_one', code, num, len(ret_1m_params))
        except Exception as _:
            log.error('get_ret_1m_one', code, str(_))
    
    tp = ThreadPool(64)
    log.info('get_ret_ann', 'begin')
    funds = db.run("select code from t_m order by code").get_rows()
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and val is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_ret_1y_ann_one, (db, fund[0]))
            tp.run(get_ret_3y_ann_one, (db, fund[0]))
            tp.run(get_ret_5y_ann_one, (db, fund[0]))
            tp.run(get_ret_6m_one, (db, fund[0]))
            tp.run(get_ret_3m_one, (db, fund[0]))
            tp.run(get_ret_1m_one, (db, fund[0]))
    tp.wait()


def get_excel():
    codes = []
    codes.append('000218')
    codes.append('000436')
    codes.append('000834')
    codes.append('005561')
    codes.append('015299')
    codes.append('016055')
    codes.append('016452')
    codes.append('016532')
    codes.append('017091')
    codes.append('018043')
    codes.append('018966')
    codes.append('019118')
    codes.append('019172')
    codes.append('019441')
    codes.append('019524')
    codes.append('019547')
    codes.append('019736')
    codes.append('040046')
    codes.append('160213')
    codes.append('161130')
    codes.append('270042')
    codes.append('539001')
    codes.sort()
    for code in codes:
        time.sleep(0.1)
        url = 'https://fund.eastmoney.com/' + code + '.html'
        text = HTTPTools.get(url, headers)
        n1 = text.split('近1年：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        # n3 = text.split('近3年：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        y1 = text.split('近1月：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        y3 = text.split('近3月：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        y6 = text.split('近6月：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        if '暂停申购' in text:
            xe = '0'
        else:
            try:
                xe = text.split('单日累计购买上限')[1].split('元<')[0]
            except:
                xe = '/'
            if str(xe).endswith('万'):
                xe = int(float(xe.replace('万', '')) * 10000)
        
        try:
            clr = text.split('成 立 日</span>：')[1].split('</td>')[0]
        except:
            clr = ''
        try:
            nhgzwc = text.split('年化跟踪误差：</a>')[1].split('%')[0] + '%'
        except:
            nhgzwc = ''
            
        # 夏普比率
        url2 = 'http://fundf10.eastmoney.com/tsdata_' + code + '.html'
        text2 = HTTPTools.get(url2, headers)
        try:
            xp1 = text2.split("夏普比率</td><td class='num'>")[1].split("</td>")[0].replace("<td class='num'>", '')
        except:
            xp1 = ''
        # try:
            # xp2 = text2.split("夏普比率</td><td class='num'>")[1].split("</td>")[1].replace("<td class='num'>", '')
        # except:
            # xp2 = ''
        # try:
            # xp3 = text2.split("夏普比率</td><td class='num'>")[1].split("</td>")[2].replace("<td class='num'>", '')
        # except:
            # xp3 = ''
            
        # 年度季度涨幅
        url3 = 'http://fundf10.eastmoney.com/FundArchivesDatas.aspx?type=jdndzf&code=' + code + '&rt=0.6097534654587713'
        text3 = HTTPTools.get(url3, headers)
        bs = BeautifulSoup(text3, features="lxml")
        trs = bs.find('table', class_='w782 comm jndxq').find('tbody').find_all('tr')
        n2025q1 = float(trs[0].find_all('td')[1].text.replace('%', '').replace('---', '0')) / 100
        n2025q2 = float(trs[0].find_all('td')[2].text.replace('%', '').replace('---', '0')) / 100
        n2025q3 = float(trs[0].find_all('td')[3].text.replace('%', '').replace('---', '0')) / 100
        n2025q4 = float(trs[0].find_all('td')[4].text.replace('%', '').replace('---', '0')) / 100
        n2025 = round(n2025q1 + n2025q2 + n2025q3 + n2025q4, 3)
        n2024q1 = float(trs[1].find_all('td')[1].text.replace('%', '').replace('---', '0')) / 100
        n2024q2 = float(trs[1].find_all('td')[2].text.replace('%', '').replace('---', '0')) / 100
        n2024q3 = float(trs[1].find_all('td')[3].text.replace('%', '').replace('---', '0')) / 100
        n2024q4 = float(trs[1].find_all('td')[4].text.replace('%', '').replace('---', '0')) / 100
        n2024 = round(n2024q1 + n2024q2 + n2024q3 + n2024q4, 3)
            
        print(str(n2024) + '\t' + str(y1) + '\t' + str(y3) + '\t' + str(y6) + '\t' + str(n1) + '\t' + str(xp1) + '\t' + str(xe))
        
        
def get_ndx(db):
    code = 'NDX'
    sql_m = 'insert into t_m(code,name,type) values(?,?,?) on conflict(code) do update set name = EXCLUDED.name, type = EXCLUDED.type'
    db.run(sql_m, ('NDX', '纳斯达克100指数', '指数'))    
    sql = 'insert into t_e(code,date,dwjz,ljjz) select ?,?,?,? where not exists(select 1 from t_e where code = ? and date = ?)'
    params = []
    start_date = db.run('select max(date) from t_e where code = ?', (code,)).get_rows()[0][0]
    end_date = Tools.locatdate()
    limit = datetime.date.today() - start_date
    url = 'https://api.nasdaq.com/api/quote/NDX/historical?assetclass=index&fromdate=' + str(start_date) + '&limit=' + str(limit.days) + '&todate=' + str(end_date) + '&lang=zh'
    text = HTTPTools.get(url, headers=headers)
    data = json.loads(text)
    rows = data['data']['tradesTable']['rows']
    for row in rows:
        day = datetime.datetime.strptime(row['date'], '%m/%d/%Y').date()
        val = float(row['close'].replace(',', ''))
        params.append((code, day, val, val, code, day))
    params.sort()
    rows = db.run(sql, params)
    log.info('get_fund_dv', code, rows, len(params))

    
def get_ndx_pe(db):
    code = 'NDX_PE'
    sql_m = 'insert into t_m(code,name,type) values(?,?,?) on conflict(code) do update set name = EXCLUDED.name, type = EXCLUDED.type'
    db.run(sql_m, (code, '纳斯达克100指数市盈率', '指数市盈率'))
    headers['cookie'] = 'acw_tc=1a0c63d917526443351605559e0085796b71f21cc037870343a4e7e1e9055c; channel=; h5_channel=; device_id=web_Sygwxj2EUgg; xq_a_token=a84b412749a831734f9e3e966ee495a5e2d4ae57; Hm_lvt_d8a99640d3ba3fdec41370651ce9b2ac=1752644342; Hm_lpvt_d8a99640d3ba3fdec41370651ce9b2ac=1752644342; HMACCOUNT=AE2B08950A2DD0B7; timestamp=1752644427208'
    text = HTTPTools.get(url='https://danjuanfunds.com/djapi/index_eva/pe_history/NDX?day=all', headers=headers)
    kvs = json.loads(text)['data']['index_eva_pe_growths']
    sql = 'insert into t_e(code,date,dwjz,ljjz) select ?,?,?,? where not exists(select 1 from t_e where code = ? and date = ?)'
    params = []
    for kv in kvs:
        date = kv['ts'] / 1000
        date = time.localtime(date)
        date = time.strftime("%Y%m%d", date)
        dwjz = kv['pe']
        params.append((code, date, dwjz, dwjz, code, date))
    params.sort()
    rows = db.run(sql, params)
    log.info('get_fund_dv', code, rows, len(params))


def backtesting(db):
    import pandas as pd
    code = ''
    yq_syl = '3.5'
    sql = '''
        select 名称
        ,(select val from t_e t2 where t2.code = t.编码 and t2.date >= ? order by date limit 1) val1
        ,(select val from t_e t2 where t2.code = t.编码 and t2.date <= ? order by date desc limit 1) val2
        ,t.* 
        from f_fund_summary(?,?,[--yq_syl--],'ret_1y_ann') t 
        where 1=1 
        and 最大单日涨幅 <= 25
        and 最大回撤 >= -10
        and 修复天数 <= 365
        and 平均收益 >= [--yq_syl--]
        and 索提诺比率 >= 1
        and 欧米茄比率 >= 1
        and 名称 not like ? 
        and 名称 not like ?     
        and 名称 not like ? 
        and 名称 not like ? 
        and 名称 not like ?
        and 名称 not like ?
        order by 索提诺比率 desc
        limit 3
    '''.replace('[--yq_syl--]', yq_syl)
    sql2 = '''
        select (select name from t_m where code = '[--code--]') name
        ,(select val from t_e where code = '[--code--]' and date >= ? order by date limit 1) val1
        ,(select val from t_e where code = '[--code--]' and date <= ? order by date desc limit 1) val2
    '''.replace('[--code--]', code)
    report = []
    all_money = 10000
    start = pd.Timestamp('2018-08-22')
    end = pd.Timestamp('2025-08-22')
    date_list = [start + pd.DateOffset(months=i) for i in range((end.year - start.year) * 12 + end.month - start.month + 1)]
    date_list = [pd.Timestamp(d) for d in date_list]
    report.append((date_list[0], all_money))
    for i in range(0, len(date_list) - 1):
        buy_day = date_list[i]
        sell_day = date_list[i + 1]
        begin_day = buy_day - relativedelta(years=6)
        end_day = buy_day
        param = (
            buy_day.strftime('%Y-%m-%d')
            , sell_day.strftime('%Y-%m-%d')
            , begin_day.strftime('%Y-%m-%d')
            , end_day.strftime('%Y-%m-%d')
            , '%美元%'
            , '%C%'
            , '%B%'
            , '%ETF'
            , '%商品%'
            , '%油%'
            # , '%指数%'
        )
        print(param)
        if code == '':
            rows = db.run(sql, param).get_rows()
        else:
            rows = db.run(sql2, (buy_day.strftime('%Y-%m-%d'), sell_day.strftime('%Y-%m-%d'))).get_rows()
        if len(rows) > 0:
            bug_money = all_money / len(rows)
            old_all_money = all_money
            all_money = 0
            for row in rows:
                # code = row[3]
                name = row[0]
                buy_price = row[1]
                sell_price = row[2]
                sell_money = bug_money / buy_price * sell_price
                all_money = all_money + sell_money
                lll = str(round((sell_money - bug_money) / bug_money * 100, 2)) + '%'
                print(buy_day.strftime('%Y-%m-%d'), sell_day.strftime('%Y-%m-%d'), name, lll, str(int(bug_money)) + '->' + str(int(sell_money)))
            lll_all_dq = str(round((all_money - old_all_money) / old_all_money * 100, 2)) + '%'
        else:
            lll_all_dq = str(round(0.02 / 12, 2)) + '%'
            old_all_money = all_money
            all_money = old_all_money * (1 + 0.02 / 12)
        lll_all = str(round((all_money - 10000) / 10000 * 100, 2)) + '%'
        print(buy_day.strftime('%Y-%m-%d'), sell_day.strftime('%Y-%m-%d'), lll_all_dq, lll_all, str(int(old_all_money)) + '->' + str(int(all_money)))
        print()
        report.append((sell_day, all_money))
        
    for r in report:
        print(r[0].strftime('%Y/%m/%d') + '\t' + str(r[1]))


if __name__ == '__main__':
    main()




# coding:utf-8
import sys
from tools_hjh import DBConn, HTTPTools
from tools_hjh import Log
from tools_hjh import ThreadPool
from tools_hjh import Tools
from bs4 import BeautifulSoup
from tools_hjh import ChromePool
import os
import time
import json

date = Tools.locatdate()
log = Log('E:/MyFiles/MyPy/log/woqukan/' + date + '.log')

host = 'https://avbebe.com'

chrome_path = r'D:\MyApps\CentBrowser\App\chrome.exe'
chromedriver_path = r'D:\MyApps\CentBrowser\chromedriver.exe'

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = 'download'
    sys_argv_1 = 'rename'
    sys_argv_1 = 'get'
    sys_argv_1 = 'get'
try:
    sys_argv_2 = sys.argv[2]
except:
    sys_argv_2 = None
try:
    sys_argv_3 = sys.argv[3]
except:
    sys_argv_3 = 2
    

def main(): 
    db = DBConn('sqlite', db='E:/MyFiles/MyPy/data/avbebe_lifan.db')
    createDatabase(db, rebuild=False)
    get_main(db, HTTPTools, '/archives/category/h動畫影片/page/[page_idx]', page_num='')
    # get_main(db, HTTPTools, '/archives/category/3d動畫/page/[page_idx]', page_num='')

    
def createDatabase(db, rebuild=False):
    if rebuild:
        db.run('drop table if exists t_m')

    t_m = '''
        create table if not exists t_m(
            url text,
            name text,
            type text,
            date text
        )
    '''
    db.run(t_m)


def get_main(db, request, firstPageUrl, page_num=''):

    def get_main_one(idx):
        try:
            params = []
            sql = 'insert into t_m select ?,?,?,? where not exists(select 1 from t_m where url = ?)'
            pageurl = host + firstPageUrl.replace('[page_idx]', str(idx))
            page = request.get(pageurl)
            bs = BeautifulSoup(page, features="lxml")
            articles = bs.find_all('article', class_='jeg_post jeg_pl_md_5 format-video')
            for article in articles:
                div_category = article.find('div', class_='jeg_post_category')
                div_content = article.find('div', class_='jeg_postblock_content')
                url = div_content.find('h3', class_='jeg_post_title').find('a')['href'].strip()
                name = div_content.find('h3', class_='jeg_post_title').find('a').text.strip()
                type_ = div_category.find('a').text
                date_ = div_content.find('div', class_='jeg_meta_date').find('a').text
                date_list = date_.split(' ')
                if len(date_list[1]) == 1:
                    dd = '0' + date_list[1]
                else:
                    dd = date_list[1]
                if len(date_list[2]) == 1:
                    mm = '0' + date_list[2]
                else:
                    mm = date_list[2]
                yyyy = date_list[4]
                date_ = yyyy + mm + dd
                params.append((url, name, type_, date_, url))
            num = db.run(sql, params)
            print(pageurl, num, len(params))
        except Exception as _:
            print(idx, _)
    
    tp = ThreadPool(16)
    
    if page_num is None or len(page_num) == 0 or page_num == '':
        pageurl = host + firstPageUrl.replace('[page_idx]', '1')
        page = request.get(pageurl)
        bs = BeautifulSoup(page, features="lxml")
        href = bs.find('a', class_='last')
        page_num = int(href.text)
    for idx in range(1, int(page_num) + 1):
        tp.run(get_main_one, (idx,))
    tp.wait()


if __name__ == '__main__':
    main()
    



# coding:utf-8
from tools_hjh.Tools import cat, echo, rm
import os
import shutil
from tools_hjh import Tools
from tools_hjh.Log import Log

date = Tools.locatdate()
log = Log('E:/MyFiles/MyPy/log/skyrim/' + date + '.log')


def main():
    mods = []
    # f_同步不同modlist的排序()
    f_把库中指定MOD全部mklink到指定路径(mods, mode='a')
    # f_把mod改名()
    # f_删除modlist中不存在的mod()
    # f_根据modlist把MOD重新分组送进文件夹()
    return


def f_根据modlist把MOD重新分组送进文件夹():
    root_path = r'E:\MyGames\SkyrimSE_MOD'
    
    mod_paths = []
    for mod_path in os.listdir(r'E:\MyGames\SkyrimSE_MOD'):
        if mod_path.startswith('mods_'):
            mod_paths.append(mod_path)
    
    modlist = cat(root_path + '\profiles\\' + '01_正常模式' + '\modlist.txt')
    lines = modlist.split('\n')
    lines.reverse()
    for line in lines:
        line = line.strip()
        if line.startswith('-') or line.startswith('+'):
            if line.endswith('_separator'):
                group_name = line.replace('_separator', '')[1:]
                continue
            else:
                mod_name = line[1:]
                for mod_path in mod_paths:
                    if mod_name in os.listdir(root_path + '\\' + mod_path):
                        src_path = root_path + '\\' + mod_path
                        dst_path = root_path + '\mods_' + group_name
                        if src_path != dst_path:
                            try:
                                Tools.mkdir(dst_path)
                            except:
                                pass
                            src_mod_path = src_path + '\\' + mod_name
                            dst_mod_path = dst_path + '\\' + mod_name
                            Tools.move_dir_for_windows(src_mod_path, dst_mod_path)
                            log.info(src_mod_path, dst_mod_path)
                        break
            # print(group_name, mod_name)


def f_删除modlist中不存在的mod():
    root_path = r'E:\MyGames\SkyrimSE_MOD'
    modlist = cat(root_path + '\profiles\\' + '01_正常模式' + '\modlist.txt')
    used_mods = []
    for line in modlist.split('\n'):
        line = line.strip()
        if len(line) > 0:
            if line[0] != '#' and not line.endswith('_separator'):
                used_mods.append(line[1:])
                
    src_paths = []
    for wjj in os.listdir(root_path):
        if wjj.startswith('mods_'):
            src_path = root_path + '\\' + wjj
            src_paths.append(src_path)
    
    for src_path in src_paths:
        for src_mod in os.listdir(src_path):
            if src_mod not in used_mods:
                log.info(src_path + '\\' + src_mod)
                rm(src_path + '\\' + src_mod)

            
def f_把库中指定MOD全部mklink到指定路径(mods=[], mode='a'): 
    root_path = r'E:\MyGames\SkyrimSE_MOD'
    need_root_path = root_path + r'\mods'
    need_root_path = 'E:\MyGames\SkyrimSE_MOD\mods'
    exists_mods = os.listdir(need_root_path)
    
    src_paths = os.listdir(root_path)

    for wjj in src_paths:
        if wjj.startswith('mods_'):
            src_path = root_path + '\\' + wjj
            for src_mod in os.listdir(src_path):
                need_flag = False
                if len(mods) > 0 and src_mod in mods:
                    need_flag = True
                elif len(mods) == 0:
                    need_flag = True
                else:
                    need_flag = False
                if need_flag:
                    if mode == 'a':
                        if src_mod not in exists_mods:
                            src_mod_path = src_path + '\\' + src_mod
                            need_mod_path = need_root_path + '\\' + src_mod
                            mklink_cmd = 'mklink /d "' + need_mod_path + '" "' + src_mod_path + '"'
                            del_cmd = 'rd /S /Q "' + need_mod_path + '"'
                            log.info(del_cmd)
                            mess = Tools.run_for_windows(del_cmd)
                            if '系统找不到指定的文件' not in mess:
                                log.info(mess)
                            log.info(mklink_cmd)
                            mess = Tools.run_for_windows(mklink_cmd)
                            log.info(mess)
                    else:
                        src_mod_path = src_path + '\\' + src_mod
                        need_mod_path = need_root_path + '\\' + src_mod
                        mklink_cmd = 'mklink /d "' + need_mod_path + '" "' + src_mod_path + '"'
                        del_cmd = 'rd /S /Q "' + need_mod_path + '"'
                        log.info(del_cmd)
                        mess = Tools.run_for_windows(del_cmd)
                        if '系统找不到指定的文件' not in mess:
                            log.info(mess)
                        log.info(mklink_cmd)
                        mess = Tools.run_for_windows(mklink_cmd)
                        log.info(mess)

                    
def f_删除库中空MOD():
    src_paths = []
    # src_paths.append(r'E:\MyGames\SkyrimSE_MOD\mods_自选')
    src_paths.append(r'E:\MyGames\SkyrimSE_MOD\mods_绝伦X5')
    src_paths.append(r'E:\MyGames\SkyrimSE_MOD\mods_季梓虞Y4')

    for src_path in src_paths:
        for src_mod in os.listdir(src_path):
            mod_path = src_path + '\\' + src_mod
            if len(os.listdir(mod_path)) == 0:
                os.rmdir(mod_path)
                print(mod_path)
            else:
                if len(os.listdir(mod_path)) == 1:
                    if os.listdir(mod_path)[0] == 'meta.ini':
                        rm(mod_path)
                        print(mod_path)

                
def f_把mod改名():
    root_path = r'E:\MyGames\SkyrimSE_MOD'
    modlist_s = []
    for profiles in os.listdir(root_path + r'\profiles'):
        modlist_path = root_path + r'\profiles' + r'\\' + profiles + r'\modlist.txt'
        modlist = cat(modlist_path)
        modlist_new = ''
        for line in modlist.split('\n'):
            line = line.strip()
            if len(line) > 0:
                modlist_new = modlist_new + line + '\n'
        modlist_s.append([modlist_path, modlist_new])
    
    mods = []
    mods.append(("爱丽西弗美化 Moolgogi Elisif", "爱丽西弗美化 Moolgogi's Elisif"))
    mods.append(("AAAAA", "BBBBB"))
    mods.append(("AAAAA", "BBBBB"))
    mods.append(("AAAAA", "BBBBB"))
    mods.append(("AAAAA", "BBBBB"))
    # mods.append(("AAAAA", "BBBBB"))
    
    src_paths = []
    for wjj in os.listdir(root_path):
        if wjj.startswith('mods_'):
            src_paths.append(root_path + r'\\' + wjj)
    
    for mod in mods:
        src_mod = mod[0]
        dst_mod = mod[1]
        
        for src_path in src_paths:
            if src_mod in os.listdir(src_path):
                need_del_mod_link = root_path + '\mods\\' + src_mod
                need_mk_mod_link = root_path + '\mods\\' + dst_mod
                src_mod_path = src_path + '\\' + src_mod
                dst_mod_path = src_path + '\\' + dst_mod
                shutil.move(src_mod_path, dst_mod_path)
                
                for i in range(0, len(modlist_s)):
                    modlist_s[i][1] = modlist_s[i][1].replace(src_mod, dst_mod)
                    
                mklink_cmd = 'mklink /d "' + need_mk_mod_link + '" "' + dst_mod_path + '"'
                del_cmd = 'rd /S /Q "' + need_del_mod_link + '"'
                log.info(del_cmd)
                mess = Tools.run_for_windows(del_cmd)
                if '系统找不到指定的文件' not in mess:
                    log.info(mess)
                log.info(mklink_cmd)
                mess = Tools.run_for_windows(mklink_cmd)
                log.info(mess)
                break
    
    for i in range(0, len(modlist_s)):
        echo(modlist_s[i][1], modlist_s[i][0], 'w')


def f_删除A文件夹中B已存在的文件():
    apath = r'E:\MyGames\SkyrimSE_MOD\mods_自选\BS 实验室'
    bpath = r'E:\MyGames\SkyrimSE_MOD\mods_自选\BS 基本'
    
    bfps = []
    for one in os.walk(bpath):
        if len(one[1]) == 0:
            for f in one[2]:
                fp = one[0] + '\\' + f
                fp = fp.replace(bpath, '')
                bfps.append(fp)
    
    for one in os.walk(apath):
        if len(one[1]) == 0:
            for f in one[2]:
                fp = one[0] + '\\' + f
                fp_ = fp.replace(apath, '')
                if fp_ in bfps:
                    rm(fp)
                    log.info(fp)


def f_删除空文件夹():
    src_path = r'E:\MyGames\SkyrimSE_MOD\mods_自选\BS 实验室'
    for src_mod in os.listdir(src_path):
        mod_path = src_path + '\\' + src_mod
        if len(os.listdir(mod_path)) == 0:
            os.rmdir(mod_path)
            log.info(mod_path)
        else:
            if len(os.listdir(mod_path)) == 1:
                if os.listdir(mod_path)[0] == 'meta.ini':
                    rm(mod_path)
                    log.info(mod_path)

                    
def f_同步不同modlist的排序():
    root_path = r'E:\MyGames\SkyrimSE_MOD'
    base_profiles = '正常模式'
    src_file = root_path + '\profiles\\' + base_profiles + r'\modlist.txt'
    src_mods = []
    for line in cat(src_file).split('\n'):
        src_mods.append(line[1:])
    
    dst_files = []
    for profile in os.listdir(root_path + r'\profiles'):
        if profile != base_profiles:
            dst_files.append(root_path + '\profiles\\' + profile + r'\modlist.txt')

    for dst_file in dst_files:
        name_flags = {}
        for line in cat(dst_file).split('\n'):
            if len(line.strip()) > 0:
                name = line[1:]
                flag = line[0]
                name_flags[name] = flag
        rs = ''
        # print(name_flags)
        for mod in src_mods:
            if len(mod.strip()) > 0:
                rs = rs + name_flags[mod.replace('DLC: ', 'DLC：')] + mod + '\n'
        echo(rs, dst_file, 'w')


if __name__ == '__main__':
    main()



# coding:utf-8
from tools_hjh.Tools import locattime, echo


def main():
    log = Log('awr_ai.log')
    log.info('a', '', 1)


class Log():
    """ 简单的日志类 """

    def __init__(self, filepath):
        self.filepath = filepath

    def info(self, *text):
        mess = ''
        for one in text:
            if one != '' and one is not None:
                mess = mess + str(one) + ', '
        mess = mess[:-2]
        mess = mess.replace('\n', '\\n')
        print(str(locattime()) + ' ' + 'info : ' + mess)
        echo(str(locattime()) + ' ' + 'info : ' + mess, self.filepath)
        
    def warning(self, *text):
        mess = ''
        for one in text:
            if one != '' and one is not None:
                mess = mess + str(one) + ', '
        mess = mess[:-2]
        mess = mess.replace('\n', '\\n')
        print(str(locattime()) + ' ' + 'warning : ' + mess)
        echo(str(locattime()) + ' ' + 'warning : ' + mess, self.filepath)
        
    def error(self, *text):
        mess = ''
        for one in text:
            if one != '' and one is not None:
                mess = mess + str(one) + ', '
        mess = mess[:-2]
        mess = mess.replace('\n', '\\n')
        print(str(locattime()) + ' ' + 'error : ' + mess)
        echo(str(locattime()) + ' ' + 'error : ' + mess, self.filepath)
        
    def out(self, *text):
        mess = ''
        for one in text:
            if one != '' and one is not None:
                mess = mess + str(one) + ', '
        mess = mess[:-2]
        print(mess)
        echo(mess, self.filepath)

        
if __name__ == '__main__': 
    main()



# coding:utf-8
import requests
import os
from tools_hjh.Tools import rm, mkdir
from urllib3 import disable_warnings
disable_warnings()


class HTTPRequest:
    """ 用户向网站提出请求的类 """
    
    def __init__(self, url, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False):
        self.url = url.strip()
        self.headers = headers
        self.data = data
        self.proxies = proxies
        self.encoding = encoding
        self.timeout = timeout
        self.stream = stream
        self.allow_redirects = allow_redirects
        self.verify = verify
        self.response = None
        self.head = None
        
        self.cookies = []
        if self.headers is not None and 'cookie' in self.headers:
            if ';' in self.headers['cookie']:
                for mess in self.headers['cookie'].split(';'):
                    self.cookies.append({'name':mess.split('=')[0].strip(), 'value':mess.split('=')[1].strip()})
            elif ';' not in self.headers['cookie'] and len(self.headers['cookie']) > 0:
                self.cookies.append({'name':self.headers['cookie'].split('=')[0].strip(), 'value':self.headers['cookie'].split('=')[1].strip()})

    def connect(self):
        """ 发出get或post请求, 返回状态码 """
        session = requests.Session()
        for cookie in self.cookies:
            session.cookies.set(cookie['name'], cookie['value'])
        if self.data is None:
            self.response = session.get(self.url, headers=self.headers, proxies=self.proxies, timeout=self.timeout, stream=self.stream, allow_redirects=self.allow_redirects, verify=self.verify)
        else:
            self.response = session.post(self.url, headers=self.headers, data=self.data, proxies=self.proxies, timeout=self.timeout, stream=self.stream, allow_redirects=self.allow_redirects, verify=self.verify)
        self.response.encoding = self.encoding
        
        return self.get_status_code()

    def get_status_code(self):
        """ 返回请求状态码 """
        if self.response is None:
            self.connect()
        return self.response.status_code
    
    def get_size(self):
        """ 返回请求大小 """
        if self.head is None:
            self.head = requests.head(self.url, headers=self.headers, data=self.data, proxies=self.proxies, timeout=self.timeout, stream=self.stream, allow_redirects=self.allow_redirects, verify=self.verify)
        return int(self.head.headers['Content-Length'])
        
    def get_text(self):
        """ 返回请求页面text """
        if self.response is None:
            self.connect()
        return self.response.text
    
    def get_content(self):
        """ 返回请求页面content """
        if self.response is None:
            self.connect()
        return self.response.content
    
    def download(self, filepath, replace=False):
        """ 下载请求的文件, 返回文件大小 """
        if self.response is None:
            self.connect()
        
        filepath = filepath.replace('\\', '/')
        
        if '/' in filepath:
            path = filepath.rsplit('/', 1)[0] + '/'
            mkdir(path)
            
        if os.path.exists(filepath) and replace:
            rm(filepath)
        elif os.path.exists(filepath) and not replace:
            return 0

        try:
            with open(filepath, 'wb') as f:
                for ch in self.response.iter_content(1024):
                    if ch:
                        f.write(ch)
        except Exception as _:
            pass
        finally:
            try:
                f.close()
            except:
                pass
        
        download_size = os.path.getsize(filepath)

        return download_size
    
    def download_until_success(self, filepath, replace=False):
        """ 下载请求的文件, 返回文件大小, 如果抛出异常就重试 """
        try:
            return self.download(filepath, replace)
        except:
            return self.download_until_success(filepath, replace)

    def close(self):
        self.response = None
        self.head = None
    
    def __del__(self):
        self.close()
    



# coding:utf-8
import sys
from tools_hjh import DBConn, Chrome
from bs4 import BeautifulSoup
from tools_hjh.HTTPTools import HTTPTools
from selenium.webdriver.common.by import By
import json
import re

chrome_path = r'D:\MyApps\CentBrowser\App\chrome.exe'
chromedriver_path = r'D:\MyApps\CentBrowser\chromedriver.exe'

headers = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
    "Referer": "https://www.bilibili.com/",
    "cookie":"buvid3=77CE9384-58A9-6FA8-2887-43BDFB8FB13153775infoc; b_nut=1738808053; _uuid=5AFD7B76-F3D5-9556-81044-310CEEB28FFBA54661infoc; buvid_fp=a707572b4a023295611d25377e5200ba; enable_web_push=DISABLE; home_feed_column=5; buvid4=B9CC925B-E9EC-FCF6-521D-CDB4DB42924F57462-025020602-nod6mU0ldP+etlR5l7OO+w==; DedeUserID=84131551; DedeUserID__ckMd5=15aa2aee95cb2c8f; hit-dyn-v2=1; header_theme_version=CLOSE; rpdid=|(JJml)mJJ~)0J'u~Jmklu~~J; enable_feed_channel=ENABLE; LIVE_BUVID=AUTO3917413704698685; PVID=4; bsource=search_bing; bmg_af_switch=1; bmg_src_def_domain=i1.hdslb.com; bili_ticket=eyJhbGciOiJIUzI1NiIsImtpZCI6InMwMyIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NDM0MjAyNjQsImlhdCI6MTc0MzE2MTAwNCwicGx0IjotMX0.hR5CWBt08cSgh5kpRlXn-lktrgPEOprhIU1mfcuY7Is; bili_ticket_expires=1743420204; SESSDATA=f46dd477,1758713389,a02d4*32CjBFIxZez9Yw4EsHKfFeEW3KUxe8ntUHbCx-84Bm5AMojY1_LZtsPCUonLkXDN8Ndf0SVkpfMmJTUGZGbXdtcG5vd29CNWhqRUZ5Y1FwckxkTW95MXlXQjl3clZkRExiNWNVb3FZU21vVXAtdkllNHBjRlhkbkpma3VBVWxCanJPdW1kWUZEellRIIEC; bili_jct=b369a500c36834d6d00fc406677d9de9; sid=7sivgz0o; b_lsid=DFCEEA9E_195DCD22A42; CURRENT_QUALITY=32; bp_t_offset_84131551=1049386541933133824; CURRENT_FNVAL=2000; browser_resolution=1536-190"
}

path = 'D:/小说视频下载/'

metadata = []
# metadata.append(('一周一次买下同班同学的那些事（有字）', 'BV1f34y1c7AM', 'MP4', '没有奶茶的世界'))
# metadata.append(('我怎么可能成为你的恋人，不行不行（有字）', 'BV1Xb4y1M7th', 'MP4', '没有奶茶的世界'))
metadata.append(('安达与岛村', 'BV1mM4y1H77M', 'MP3', '没有奶茶的世界'))
# metadata.append(('家里蹲吸血姬的郁闷', 'BV1tF411Z7BM', 'MP3', '没有奶茶的世界'))
# metadata.append(('转生王女与天才千金的魔法革命', 'BV1oN411a7oW', 'MP3', '没有奶茶的世界'))
# metadata.append(('将放言说女生之间不可能的女孩子，在百日之内彻底攻陷的百合故事', 'BV1yk4y1Z7G7', 'MP3', '没有奶茶的世界'))

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = 'get'
    sys_argv_1 = 'print'
    sys_argv_1 = 'rename'
    sys_argv_1 = 'get'
try:
    sys_argv_2 = sys.argv[2]
except:
    sys_argv_2 = None
try:
    sys_argv_3 = sys.argv[3]
except:
    sys_argv_3 = 32
    

def main(): 
    db = DBConn('sqlite', db='D:/MyFiles/MyPy/data/bilibili.db')
    createDatabase(db, rebuild=True)
    if  sys_argv_1 == 'get':
        get(db)

    
def createDatabase(db, rebuild=False):
    if rebuild:
        db.run('drop table if exists t_m')

    t_m = '''
        create table if not exists t_m(
            name_ text,
            uploader text,
            url text, 
            title text,
            definition text,
            downloaded text
        )
    '''
    
    db.run(t_m)

    
def get(db):
    insert_sql = 'insert into t_m select ?,?,?,?,?,? where not exists(select 1 from t_m where url = ?)'
    cp = Chrome(chrome_path=chrome_path, chromedriver_path=chromedriver_path, is_hidden=False, is_display_picture=False, headers=headers)
    for meta in metadata:
        name_ = meta[0]
        id_ = meta[1]
        type_ = meta[2]
        up_ = meta[3]
        
        page = cp.get('https://www.bilibili.com/video/' + id_)
        volumes = cp.chrome.find_elements(By.CLASS_NAME, 'slide-item')
        
        i = 1
        params = []
        for volume in volumes:
            try:
                volume.click()
            except:
                continue
            page = cp.chrome.page_source
            bs = BeautifulSoup(page, features="lxml")
            videos = bs.find('div', class_='video-pod__body').find_all('div', class_='pod-item video-pod__item simple')
            for video in videos:
                if i <= 9:
                    tt = '0' + str(i)
                else:
                    tt = str(i)
                url = 'https://www.bilibili.com/video/' + video['data-key']
                title = tt + '_' + video.find('div', class_='title-txt').text.replace(' ', '_')
                
                v_page = HTTPTools.get(url, headers=headers)
                script_content = re.search(r'<script>window\.__playinfo__=(.*?)</script>', v_page).group(1)
                video_info = json.loads(script_content)
                
                audio_url = video_info['data']['dash']['audio'][0]['baseUrl']
                params.append((name_, up_, audio_url, title + '.mp3', '0P', '0', url))
                print(audio_url)
                
                if type_ == 'MP4':
                    
                    param = {}
                    for v in video_info['data']['dash']['video']:
                        if v['height'] == 720:
                            param['720'] = (name_, up_, v['baseUrl'], title + '.mp4', '720P', '0', url)
                        elif v['height'] == 480:
                            param['480'] = (name_, up_, v['baseUrl'], title + '.mp4', '480P', '0', url)
                        elif v['height'] == 1080:
                            param['1080'] = (name_, up_, v['baseUrl'], title + '.mp4', '1080P', '0', url)
                        elif v['height'] == 360:
                            param['360'] = (name_, up_, v['baseUrl'], title + '.mp4', '360P', '0', url)
                            
                    if '720' in param:
                        params.append(param['720'])
                        print(name_, title + '.mp4', '720P')
                    elif '720' not in param and '480' in param:
                        params.append(param['480'])
                        print(name_, title + '.mp4', '480P')
                    elif '720' not in param and '480' not in param and '1080' in param:
                        params.append(param['1080'])
                        print(name_, title + '.mp4', '1080P')
                    else:
                        params.append((name_, up_, video_info['data']['dash']['video'][0]['baseUrl'], title + '.mp4', str(video_info['data']['dash']['video']['height']) + 'P', '0', url))
                        print(name_, title + '.mp4', '360P')

                i = i + 1
                db.run(insert_sql, params)
                params.clear()
    cp.close()
    
    
if __name__ == '__main__':
    main()




# coding:utf-8
from uuid import uuid1
from concurrent.futures import ThreadPoolExecutor, wait
import time


def main():
    pass


class ThreadPool():
    """ 维护一个线程池 """
    
    def __init__(self, size):
        self.size = size
        self.pool = ThreadPoolExecutor(max_workers=self.size)
        self.task_dict = {}
        
    def run(self, func, args=(), kwargs={}, name=None):
        id_ = uuid1()
        task = self.pool.submit(func, *args, **kwargs)
        self.task_dict[id_] = (name, task)
        return id_
    
    def run_wait(self, func, args=(), kwargs={}, name=None):
        while self.get_running_num() >= self.size:
            time.sleep(0.1)
        self.run(func, args, kwargs, name)

    def get_results(self, timeout=None):
        self.wait()
        task_rs = {}
        for k, v in self.task_dict.items():
            task_rs[k] = v[1].result(timeout=timeout)
        return task_rs
    
    def get_result(self, id_, timeout=None):
        return self.task_dict[id_][1].result(timeout=timeout)

    def wait(self, timeout=None):
        tasks = []
        for _, v in self.task_dict.items():
            tasks.append(v[1])
        wait(tasks, timeout=timeout)
    
    def get_running_num(self):
        running_tasks_num = 0
        for _, v in self.task_dict.items():
            if not v[1].done():
                running_tasks_num = running_tasks_num + 1
        return running_tasks_num
    
    def get_running_name(self):
        running_task_names = []
        for _, v in self.task_dict.items():
            if not v[1].done():
                running_task_names.append(v[0])
        return running_task_names
    
    def close(self):
        self.pool.shutdown(wait=False)


if __name__ == '__main__':
    main()



# coding:utf-8
import sys
from tools_hjh import DBConn
from tools_hjh import Log
from tools_hjh import ThreadPool
from tools_hjh import Tools
from tools_hjh import HTTPTools
import json
import time
import datetime
from dateutil.relativedelta import relativedelta
import numpy as np
import math
from tools_hjh.Tools import str_exception
from bs4 import BeautifulSoup

stop_flag = False
page_num = None

date = Tools.locatdate()
log = Log('E:/MyFiles/MyPy/log/fund/' + date + '.log')

host = 'https://'

chrome_path = r'D:\MyApps\CentBrowser\App\chrome.exe'
chromedriver_path = r'D:\MyApps\CentBrowser\chromedriver.exe'

headers = {
    'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
    'accept-encoding': 'text',
    'accept-language': 'zh-CN,zh;q=0.9',
    'cache-control': 'no-cache',
    'dnt': '1',
    'pragma': 'no-cache',
    'sec-ch-ua': 'x86;v=99, Windows;v=10, Surface Laptop Studio;v=1',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-platform': '"Windows"',
    'Referer':'http://fundf10.eastmoney.com/',
    'user-agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.101 Safari/537.36',
    'cookie':'qgqp_b_id=5b3011884880cd251f1bc065e16256ae; emshistory=%5B%22513100%22%5D; EMFUND1=null; EMFUND2=null; EMFUND3=null; EMFUND4=null; EMFUND5=null; EMFUND6=null; HAList=ty-103-NQ00Y-%u5C0F%u578B%u7EB3%u6307%u5F53%u6708%u8FDE%u7EED%2Cty-100-NDX100-%u7EB3%u65AF%u8FBE%u514B100%2Cty-133-USDCNH-%u7F8E%u5143%u79BB%u5CB8%u4EBA%u6C11%u5E01%2Cty-1-512690-%u9152ETF%2Cty-1-513100-%u7EB3%u6307ETF%2Cty-105-NVDA-%u82F1%u4F1F%u8FBE%2Cty-1-600900-%u957F%u6C5F%u7535%u529B%2Cty-1-000001-%u4E0A%u8BC1%u6307%u6570; st_si=04690614705089; st_asi=delete; EMFUND0=null; EMFUND7=07-18%2022%3A48%3A53@%23%24%u62DB%u5546%u4E2D%u8BC1%u767D%u9152%u6307%u6570%28LOF%29A@%23%24161725; EMFUND8=07-19%2002%3A43%3A47@%23%24%u6613%u65B9%u8FBE%u5929%u5929%u7406%u8D22%u8D27%u5E01A@%23%24000009; EMFUND9=07-29 10:05:57@#$%u534E%u590F%u6210%u957F%u6DF7%u5408@%23%24000001; st_pvi=99805886133314; st_sp=2024-04-24%2009%3A56%3A14; st_inirUrl=http%3A%2F%2Fquote.eastmoney.com%2Fsh513100.html; st_sn=4; st_psi=20240729100557782-112200305282-3955411629'
}

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = 'download'
try:
    sys_argv_2 = int(sys.argv[2])
except:
    sys_argv_2 = -1  # 获取条数
try:
    sys_argv_3 = sys.argv[3]
except:
    sys_argv_3 = 1
    
thread_num = 64


def main(): 
    db = DBConn('sqlite', db='E:/MyFiles/MyPy/data/fund.db', poolsize=64)
    try:
        db.run('PRAGMA journal_mode = WAL')
        createDatabase(db, rebuild=False)
        get_fund_main(db, HTTPTools)
        # get_fund_dv(db, HTTPTools, begin_code='000000', code=None, name_like='%国泰纳斯达克100指数%')
        # get_fund_dv(db, HTTPTools, begin_code='000000', code=None, name_like='%纳斯达克%')
        # get_fund_dv(db, HTTPTools, begin_code='000000', code=None, name_like='%纳指%')
        # fix_fund_dv(db, HTTPTools, begin_code='000000', code=None, name_like='%%')
        get_fund_dv(db, HTTPTools, begin_code='000000', code=None, name_like='%%')
        # get_ndx(r'E:\Users\HJH\下载\纳斯达克100指数历史数据 (4).csv')
        fix_dwjz(db)
        get_zf(db)
        get_val(db)
        get_hc(db)
        get_nh(db)
        get_xp(db)
    finally:
        db.run('PRAGMA main.wal_checkpoint')
        db.close()


def createDatabase(db, rebuild=False):
    if rebuild:
        db.run('drop table if exists t_m')
        db.run('drop table if exists t_e')

    t_m = '''
        create table if not exists t_m(
            code char(9),
            name varchar(255),
            type varchar(255),
            primary key(code)
        )
    '''
    
    t_e = '''
        create table if not exists t_e(
            code char(9),
            date char(10),
            dwjz real,
            ljjz real,
            zf real,
            fh text,
            val real,
            hc real,
            nh1n real,
            nh3n real,
            nh5n real,
            xp1n real,
            xp3n real,
            xp5n real,
            primary key(code, date)
        )
    '''
    
    db.run(t_m)
    db.run(t_e)


def get_fund_main(db, rep):
    sql = 'insert or replace into t_m(code,name,type) values(?,?,?)'
    page = rep.get('http://fund.eastmoney.com/js/fundcode_search.js')
    page = page.encode('utf8')[3:].decode('utf8')
    page = page.replace('var r = ', '').replace(';', '')
    fund_list = json.loads(page)
    params = []
    for fund in fund_list:
        code = fund[0]
        name = fund[2]
        type_ = fund[3]
        params.append((code, name, type_))
    num = db.run(sql, params)
    log.info('get_fund_main', num)


# 从库中最大的date开始，获取至今的数据，如果中间缺了不会补
def get_fund_dv(db, rep, begin_code='000000', code=None, name_like=''):
            
    sql = "replace into t_e(code,date,dwjz,ljjz,fh) values(?,?,?,?,?)"
    vals_all = []

    def get_fund_dv_one(code, rep, times=1):
        try:
            timestamp = str(int(time.time() * 1000))
            end_date = Tools.locatdate()
            start_date = db.run('select max(date) from t_e where code = ?', (code,)).get_rows()[0][0]
            if start_date is None:
                start_date = '1949-10-01'
            
            if end_date == start_date:
                log.info('get_fund_dv_one', code, 0, len(vals_all), times)
                return
            
            page_size = 20
            all_size = (datetime.datetime.strptime(end_date, "%Y-%m-%d").date() - datetime.datetime.strptime(start_date, "%Y-%m-%d").date()).days
            # all_size = 230
            page_num = math.ceil(all_size / page_size)
            
            rss_num = 0
            for page_idx in range(1, page_num + 1):
                url = 'http://api.fund.eastmoney.com/f10/lsjz?callback=jQuery18306743973867400965_1722217638986&fundCode=' + code + '&pageIndex=' + str(page_idx) + '&pageSize=' + str(page_size) + '&startDate=' + start_date + '&endDate=' + end_date + '&_=' + timestamp
                page = rep.get(url, headers=headers)
                page = page.replace('jQuery18306743973867400965_1722217638986', '').strip()
                page = page.split('"LSJZList":')[1].split(',"FundType"')[0]
                rss = json.loads(page)
                rss_num = rss_num + len(rss)
                
                if len(rss) == 0:
                    break
                
                for rs in reversed(rss):
                    date_ = rs['FSRQ']
                    dwjz = rs['DWJZ']
                    if dwjz == '':
                        dwjz = None
                    ljjz = rs['LJJZ']
                    if ljjz == '':
                        ljjz = None
                    zf = rs['JZZZL']
                    if zf == '':
                        zf = 0
                    fh = rs['FHSP']
                    if fh == '':
                        fh = None
         
                    vals_all.append((code, date_, dwjz, ljjz, fh))

            log.info('get_fund_dv_one', code, rss_num, len(vals_all), times)
                    
        except Exception as _:
            if times <= 99:
                # log.warning('get_fund_dv_one', code, times, _)
                # time.sleep(1)
                times = times + 1
                get_fund_dv_one(code, rep, times)
            else:
                log.error('get_fund_dv_one', code, times, str_exception(_))
    
    if code is not None:
        get_code_sql = '''
            select code from t_m 
            where code = ?
        '''
        funds = db.run(get_code_sql, (begin_code,)).get_rows()
    else:
        get_code_sql = '''
            select code from t_m 
            where 1=1
            and type not in('货币型-普通货币')
            and code >= ?
            and name like ?
            order by code
        '''
        funds = db.run(get_code_sql, (begin_code, name_like)).get_rows()
    
    tp = ThreadPool(thread_num)
    for fund in funds:
        if len(vals_all) >= 1000000:
            tp.wait()
            num = db.run(sql, vals_all)
            log.info('get_fund_dv', num)
            vals_all.clear()
        else:
            code = fund[0]
            tp.run_wait(get_fund_dv_one, (code, rep))
            
    tp.wait()
    num = db.run(sql, vals_all)
    log.info('get_fund_dv', num)
    vals_all.clear()


# 从库中最大的date开始，获取至今的数据，如果中间缺了不会补
def fix_fund_dv(db, rep, begin_code='000000', code=None, name_like=''):
            
    sql = "replace into t_e(code,date,dwjz,ljjz,fh) values(?,?,?,?,?)"
    vals_all = []

    def fix_fund_dv_one(code, rep, times=1):
        try:
            idx = 1
            page_size = 9999
            timestamp = str(int(time.time() * 1000))
            start_date = '1949-10-01'
            db_count = int(db.run('select count(1) from t_e where code = ?', (code,)).get_rows()[0][0])
            try:
                end_date = db.run('select max(date) from t_e where code = ?', (code,)).get_rows()[0][0]
            except:
                end_date = Tools.locatdate()
            if end_date is None:
                end_date = Tools.locatdate()
            
            url = 'http://api.fund.eastmoney.com/f10/lsjz?callback=jQuery18306743973867400965_1722217638986&fundCode=' + code + '&pageIndex=' + str(idx) + '&pageSize=' + str(page_size) + '&startDate=' + start_date + '&endDate=' + end_date + '&_=' + timestamp
            page = rep.get(url, headers=headers)
            page = page.replace('jQuery18306743973867400965_1722217638986', '').strip()
            page = page.split('"LSJZList":')[1].split(',"FundType"')[0]
            rss = json.loads(page)
            
            if len(rss) == 0 or len(rss) == db_count:
                log.info('fix_fund_dv_one', code, 0, len(vals_all), times)
                return
            
            for rs in reversed(rss):
                date_ = rs['FSRQ']
                dwjz = rs['DWJZ']
                if dwjz == '':
                    dwjz = None
                ljjz = rs['LJJZ']
                if ljjz == '':
                    ljjz = None
                zf = rs['JZZZL']
                if zf == '':
                    zf = 0
                fh = rs['FHSP']
                if fh == '':
                    fh = None
     
                vals_all.append((code, date_, dwjz, ljjz, fh))

            log.info('fix_fund_dv_one', code, len(rss), len(vals_all), times)
                    
        except Exception as _:
            if times <= 99:
                # log.warning('fix_fund_dv_one', code, times, _)
                # time.sleep(1)
                times = times + 1
                fix_fund_dv_one(code, rep, times)
            else:
                log.error('fix_fund_dv_one', code, times, _)
    
    if code is not None:
        get_code_sql = '''
            select code from t_m 
            where code = ?
        '''
        funds = db.run(get_code_sql, (begin_code,)).get_rows()
    else:
        get_code_sql = '''
            select code from t_m 
            where 1=1
            and type not in('货币型-普通货币')
            and code >= ?
            and name like ?
            order by code
        '''
        funds = db.run(get_code_sql, (begin_code, name_like)).get_rows()
    
    tp = ThreadPool(thread_num)
    for fund in funds:
        if len(vals_all) >= 1000000:
            tp.wait()
            num = db.run(sql, vals_all)
            log.info('fix_fund_dv', num)
            vals_all.clear()
        else:
            code = fund[0]
            tp.run_wait(fix_fund_dv_one, (code, rep))
            
    tp.wait()
    num = db.run(sql, vals_all)
    log.info('fix_fund_dv', num)
    vals_all.clear()


def fix_dwjz(db):

    def get_dwjz_one(db, code):
        sql = 'update t_e set dwjz = ? where code = ? and date = ?'
        params = []
        rss = db.run('select date,fh from t_e where code = ? and dwjz is null order by date', (code,)).get_rows()
        for rs in rss:
            date_ = rs[0]
            fh = rs[1]
                
            try:
                previous_dwjz = float(db.run('select dwjz from t_e where code = ? and date < ? and dwjz is not null order by date desc limit 1', (code, date_)).get_rows()[0][0])
                if previous_dwjz is None:
                    previous_dwjz = 1
            except:
                previous_dwjz = 1 
        
            if '每份基金份额分拆' in str(fh):
                fh_cf = float(fh.replace('每份基金份额分拆', '').replace('份', ''))
                dwjz = previous_dwjz / fh_cf
            elif '每份派现金' in str(fh):
                fh_xj = float(fh.replace('每份派现金', '').replace('元', ''))
                dwjz = previous_dwjz - fh_xj
            else:
                dwjz = previous_dwjz
                
            params.append((dwjz, code, date_))
            
        num = db.run(sql, params)
        log.info('get_dwjz', code, num, len(params))

    tp = ThreadPool(64)
    log.info('get_dwjz', 'begin')
    funds = db.run("select code from t_m order by code").get_rows()
    funds.append(('NDX',))
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ?', (fund[0],)).get_rows()[0][0]
        if num > 0:
            tp.run(get_dwjz_one, (db, fund[0]))
    tp.wait()


def get_zf(db):

    def get_zf_one(db, code):
        sql = 'update t_e set zf = ? where code = ? and date = ?'
        params = []
        rss = db.run('select date,dwjz,fh from t_e where code = ? and zf is null order by date', (code,)).get_rows()
        for rs in rss:
            date_ = rs[0]
            dwjz = float(rs[1])
            fh = rs[2]
            try:
                previous_dwjz = float(db.run('select dwjz from t_e where code = ? and date < ? order by date desc limit 1', (code, date_)).get_rows()[0][0])
            except:
                zf = 0
                params.append((zf, code, date_))
                continue
            
            if '每份基金份额分拆' in str(fh):
                fh_cf = float(fh.replace('每份基金份额分拆', '').replace('份', ''))
                zf = ((dwjz * fh_cf) - previous_dwjz) / previous_dwjz * 100
            elif '每份派现金' in str(fh):
                fh_xj = float(fh.replace('每份派现金', '').replace('元', ''))
                zf = ((dwjz + fh_xj) - previous_dwjz) / previous_dwjz * 100
            elif '每份基金份额折算' in str(fh):
                fh_zs = float(fh.replace('每份基金份额折算', '').replace('份', ''))
                zf = (dwjz - (previous_dwjz / fh_zs)) / (previous_dwjz / fh_zs) * 100
            else:
                zf = (dwjz - previous_dwjz) / previous_dwjz * 100
            
            params.append((zf, code, date_))
        try:
            num = db.run(sql, params)
            log.info('get_zf', code, num, len(params))
        except Exception as _:
            log.error('get_zf', code, str(_))

    tp = ThreadPool(64)
    log.info('get_zf', 'begin')
    funds = db.run("select code from t_m").get_rows()
    funds.append(('NDX',))
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and dwjz is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_zf_one, (db, fund[0]))
    tp.wait()

    
def get_val(db):

    def get_val_one(db, code):
        sql = 'update t_e set val = ? where code = ? and date = ?'
        params = []   
        rss = db.run('select date,zf from t_e where code = ? and val is null order by date', (code,)).get_rows()
        next_val = 1 
        for rs in rss:
            try:
                date_ = rs[0]
                zf = rs[1]
                try:
                    next_val_db = None
                    next_val_db = db.run('select val from t_e where code = ? and date < ? order by date desc limit 1', (code, date_)).get_rows()[0][0]
                except:
                    next_val_db = None
                if next_val_db is not None:
                    next_val = next_val_db
                val = float(next_val) * (1 + float(zf) / 100)
                params.append((val, code, date_))
                next_val = val
            except Exception as _:
                print(_, code, date_, next_val, zf)
        try:
            num = db.run(sql, params)
            log.info('get_val', code, num, len(params))
        except Exception as _:
            log.error('get_val', code, str(_))

    tp = ThreadPool(64)
    log.info('get_val', 'begin')
    funds = db.run("select code from t_m").get_rows()
    funds.append(('NDX',))
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and zf is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_val_one, (db, fund[0]))
    tp.wait()


def get_hc(db):

    def get_hc_one(db, code):
        params = []      
        sql = 'update t_e set hc = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where hc is null and code = ?', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            val = row[1]
            max_val = db.run('select max(val) from t_e where date <= ? and code = ?', (date_now, code)).get_rows()[0][0]
            val = float(val)
            max_val = float(max_val)
            hc = round((max_val - val) / max_val * 100, 3)
            params.append((hc, code, date_now))
        try:
            num = db.run(sql, params)
            log.info('get_hc', code, num, len(params))
        except Exception as _:
            log.error('get_hc', code, str(_))
    
    tp = ThreadPool(64)
    log.info('get_hc', 'begin')
    funds = db.run("select code from t_m").get_rows()
    funds.append(('NDX',))
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and val is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_hc_one, (db, fund[0]))
    tp.wait()


def get_nh(db):

    def get_nh1n_one(db, code):
        nh1n_params = []
        nh1n_sql = 'update t_e set nh1n = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where nh1n is null and code = ?', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_future = (datetime.datetime.strptime(date_now, '%Y-%m-%d') + relativedelta(years=1)).strftime('%Y-%m-%d')
            val = row[1]
            val = float(val)
            try:
                val_future = db.run('select val from t_e where date >= ? and code = ? order by date limit 1', (date_future, code)).get_rows()[0][0]
                val_future = float(val_future)
                nh1n = round((val_future - val) / val * 100, 3)
            except:
                nh1n = None
            if nh1n is None:
                break
            nh1n_params.append((nh1n, code, date_now))
        try:
            num = db.run(nh1n_sql, nh1n_params)
            log.info('get_nh1n_one', code, num, len(nh1n_params))
        except Exception as _:
            log.error('get_nh1n_one', code, str(_))
        
    def get_nh3n_one(db, code):
        nh3n_params = []
        nh3n_sql = 'update t_e set nh3n = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where nh3n is null and code = ?', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_future = (datetime.datetime.strptime(date_now, '%Y-%m-%d') + relativedelta(years=3)).strftime('%Y-%m-%d')
            val = row[1]
            val = float(val)
            try:
                val_future = db.run('select val from t_e where date >= ? and code = ? order by date limit 1', (date_future, code)).get_rows()[0][0]
                val_future = float(val_future)
                nh3n = round((val_future - val) / val / 3 * 100, 3)
            except:
                nh3n = None
            if nh3n is None:
                break
            nh3n_params.append((nh3n, code, date_now))
        try:
            num = db.run(nh3n_sql, nh3n_params)
            log.info('get_nh3n_one', code, num, len(nh3n_params))
        except Exception as _:
            log.error('get_nh3n_one', code, str(_))
        
    def get_nh5n_one(db, code):
        nh5n_params = []
        nh5n_sql = 'update t_e set nh5n = ? where code = ? and date = ?'
        rows = db.run('select date,val from t_e where nh5n is null and code = ?', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_future = (datetime.datetime.strptime(date_now, '%Y-%m-%d') + relativedelta(years=5)).strftime('%Y-%m-%d')
            val = row[1]
            val = float(val)
            try:
                val_future = db.run('select val from t_e where date >= ? and code = ? order by date limit 1', (date_future, code)).get_rows()[0][0]
                val_future = float(val_future)
                nh5n = round((val_future - val) / val / 5 * 100, 3)
            except:
                nh5n = None
            if nh5n is None:
                break
            nh5n_params.append((nh5n, code, date_now))
        try:
            num = db.run(nh5n_sql, nh5n_params)
            log.info('get_nh5n_one', code, num, len(nh5n_params))
        except Exception as _:
            log.error('get_nh5n_one', code, str(_))
    
    tp = ThreadPool(64)
    log.info('get_nh', 'begin')
    funds = db.run("select code from t_m").get_rows()
    funds.append(('NDX',))
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and val is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_nh1n_one, (db, fund[0]))
            tp.run(get_nh3n_one, (db, fund[0]))
            tp.run(get_nh5n_one, (db, fund[0]))
    tp.wait()
    

def get_xp(db):

    def get_xp1n_one(db, code):
        xp1n_params = []
        xp1n_sql = 'update t_e set xp1n = ? where code = ? and date = ?'
        rows = db.run('select date from t_e where xp1n is null and code = ?', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_future = (datetime.datetime.strptime(date_now, '%Y-%m-%d') + relativedelta(years=1)).strftime('%Y-%m-%d')
            try:
                zfs_future_num = db.run('select count(1) from t_e where date >= ? and code = ? order by date limit 1', (date_future, code)).get_rows()[0][0]
                if zfs_future_num == 0:
                    break
                zfs_future = db.run('select zf from t_e where date >= ? and date < ? and code = ? order by date', (date_now, date_future, code)).get_rows()
                zfs_future = [c[0] / 100 for c in zfs_future]
                year_rate = sum(zfs_future)
                day_standard_deviation = np.std(zfs_future) * math.sqrt(365)
                if day_standard_deviation == 0:
                    xp1n = None
                else:
                    xp1n = round(year_rate / day_standard_deviation, 3)
            except Exception as _:
                xp1n = None
            xp1n_params.append((xp1n, code, date_now))
        try:
            num = db.run(xp1n_sql, xp1n_params)
            log.info('get_xp1n_one', code, num, len(xp1n_params))
        except Exception as _:
            log.error('get_xp1n_one', code, str(_))
        
    def get_xp3n_one(db, code):
        xp3n_params = []
        xp3n_sql = 'update t_e set xp3n = ? where code = ? and date = ?'
        rows = db.run('select date from t_e where xp3n is null and code = ?', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_future = (datetime.datetime.strptime(date_now, '%Y-%m-%d') + relativedelta(years=3)).strftime('%Y-%m-%d')
            try:
                zfs_future_num = db.run('select count(1) from t_e where date >= ? and code = ? order by date limit 1', (date_future, code)).get_rows()[0][0]
                if zfs_future_num == 0:
                    break
                zfs_future = db.run('select zf from t_e where date >= ? and date < ? and code = ? order by date', (date_now, date_future, code)).get_rows()
                zfs_future = [c[0] / 100 for c in zfs_future]
                year_rate = sum(zfs_future)
                day_standard_deviation = np.std(zfs_future) * math.sqrt(365 * 3)
                if day_standard_deviation == 0:
                    xp3n = None
                else:
                    xp3n = round(year_rate / day_standard_deviation, 3)
            except Exception as _:
                xp3n = None
            xp3n_params.append((xp3n, code, date_now))
        try:
            num = db.run(xp3n_sql, xp3n_params)
            log.info('get_xp3n_one', code, num, len(xp3n_params))
        except Exception as _:
            log.error('get_xp3n_one', code, str(_))
        
    def get_xp5n_one(db, code):
        xp5n_params = []
        xp5n_sql = 'update t_e set xp5n = ? where code = ? and date = ?'
        rows = db.run('select date from t_e where xp5n is null and code = ?', (code,)).get_rows()
        for row in rows:
            date_now = row[0]
            date_future = (datetime.datetime.strptime(date_now, '%Y-%m-%d') + relativedelta(years=5)).strftime('%Y-%m-%d')
            try:
                zfs_future_num = db.run('select count(1) from t_e where date >= ? and code = ? order by date limit 1', (date_future, code)).get_rows()[0][0]
                if zfs_future_num == 0:
                    break
                zfs_future = db.run('select zf from t_e where date >= ? and date < ? and code = ? order by date', (date_now, date_future, code)).get_rows()
                zfs_future = [c[0] / 100 for c in zfs_future]
                year_rate = sum(zfs_future)
                day_standard_deviation = np.std(zfs_future) * math.sqrt(365 * 5)
                if day_standard_deviation == 0:
                    xp5n = None
                else:
                    xp5n = round(year_rate / day_standard_deviation, 3)
            except Exception as _:
                xp5n = None
            xp5n_params.append((xp5n, code, date_now))
        try:
            num = db.run(xp5n_sql, xp5n_params)
            log.info('get_xp5n_one', code, num, len(xp5n_params))
        except Exception as _:
            log.error('get_xp5n_one', code, str(_))
    
    tp = ThreadPool(64)
    log.info('get_xp', 'begin')
    funds = db.run("select code from t_m").get_rows()
    funds.append(('NDX',))
    for fund in funds:
        num = db.run('select count(1) from t_e where code = ? and zf is null', (fund[0],)).get_rows()[0][0]
        if num == 0:
            tp.run(get_xp1n_one, (db, fund[0]))
            tp.run(get_xp3n_one, (db, fund[0]))
            tp.run(get_xp5n_one, (db, fund[0]))
    tp.wait()


def fun1():
    codes = ['000218', '019118', '012752', '006479', '000834', '015299', '016055', '016452', '016532', '017091', '018043', '018966', '019172', '019441', '019524', '019547', '019736', '040046', '160213', '161130', '270042', '539001']
    # codes = ['019524']
    codes.sort()
    for code in codes:
        url = 'https://fund.eastmoney.com/' + code + '.html'
        text = HTTPTools.get(url, headers)
        n1 = text.split('近1年：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        n3 = text.split('近3年：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        y1 = text.split('近1月：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        y3 = text.split('近3月：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        y6 = text.split('近6月：')[1].split('ui-font-middle')[1].split('ui-num">')[1].split('<')[0]
        try:
            xe = text.split('单日累计购买上限')[1].split('元<')[0]
            if int(xe) >= 100000:
                xe = ''
        except:
            xe = ''
        if str(xe).endswith('万'):
            xe = int(float(xe.replace('万', '')) * 10000)
            
        try:
            clr = text.split('成 立 日</span>：')[1].split('</td>')[0]
        except:
            clr = ''
        try:
            nhgzwc = text.split('年化跟踪误差：</a>')[1].split('%')[0] + '%'
        except:
            nhgzwc = ''
            
        # 夏普比率
        url2 = 'http://fundf10.eastmoney.com/tsdata_' + code + '.html'
        text2 = HTTPTools.get(url2, headers)
        try:
            xp1 = text2.split("夏普比率</td><td class='num'>")[1].split("</td>")[0].replace("<td class='num'>", '')
        except:
            xp1 = ''
        try:
            xp2 = text2.split("夏普比率</td><td class='num'>")[1].split("</td>")[1].replace("<td class='num'>", '')
        except:
            xp2 = ''
        try:
            xp3 = text2.split("夏普比率</td><td class='num'>")[1].split("</td>")[2].replace("<td class='num'>", '')
        except:
            xp3 = ''
            
        # 年度季度涨幅
        url3 = 'http://fundf10.eastmoney.com/FundArchivesDatas.aspx?type=jdndzf&code=' + code + '&rt=0.6097534654587713'
        text3 = HTTPTools.get(url3, headers)
        bs = BeautifulSoup(text3, features="lxml")
        trs = bs.find('table', class_='w782 comm jndxq').find('tbody').find_all('tr')
        n2025q1 = float(trs[0].find_all('td')[1].text.replace('%', '').replace('---', '0')) / 100
        n2025q2 = float(trs[0].find_all('td')[2].text.replace('%', '').replace('---', '0')) / 100
        n2025q3 = float(trs[0].find_all('td')[3].text.replace('%', '').replace('---', '0')) / 100
        n2025q4 = float(trs[0].find_all('td')[4].text.replace('%', '').replace('---', '0')) / 100
        n2025 = round(n2025q1 + n2025q2 + n2025q3 + n2025q4, 3)
        n2024q1 = float(trs[1].find_all('td')[1].text.replace('%', '').replace('---', '0')) / 100
        n2024q2 = float(trs[1].find_all('td')[2].text.replace('%', '').replace('---', '0')) / 100
        n2024q3 = float(trs[1].find_all('td')[3].text.replace('%', '').replace('---', '0')) / 100
        n2024q4 = float(trs[1].find_all('td')[4].text.replace('%', '').replace('---', '0')) / 100
        n2024 = round(n2024q1 + n2024q2 + n2024q3 + n2024q4, 3)
            
        print(str(n2025) + '\t' + str(n2024) + '\t' + str(n1) + '\t' + str(xp1) + '\t' + str(clr) + '\t' + str(nhgzwc))

        
def get_ndx(path):
    db = DBConn('sqlite', db='E:/MyFiles/MyPy/data/fund.db', poolsize=64)
    text = Tools.cat(path)
    sql = 'insert into t_e(code,date,dwjz,ljjz) select ?,?,?,? where not exists(select 1 from t_e where code = ? and date = ?)'
    params = []
    for line in text.split('\n')[1:]:
        if len(line.strip()) > 0:
            code = 'NDX'
            date0 = line.split('","')[0].replace('"', '').split('-')[0]
            date1 = line.split('","')[0].replace('"', '').split('-')[1]
            if len(date1) == 1:
                date1 = '0' + date1
            date2 = line.split('","')[0].replace('"', '').split('-')[2]
            if len(date2) == 1:
                date2 = '0' + date2
            date = date0 + '-' + date1 + '-' + date2
            if len(date2) == 1:
                date2 = '0' + date2
            dwjz = float(line.split('","')[1].replace('"', '').replace(',', ''))
            params.append((code, date, dwjz, dwjz, code, date))
    params.sort()
    rows = db.run(sql, params)
    print('NDX', rows)

    
def get_ndx_pe():
    db = DBConn('sqlite', db='E:/MyFiles/MyPy/data/fund.db', poolsize=64)
    headers['cookie'] = 'acw_tc=1a0c63d917526443351605559e0085796b71f21cc037870343a4e7e1e9055c; channel=; h5_channel=; device_id=web_Sygwxj2EUgg; xq_a_token=a84b412749a831734f9e3e966ee495a5e2d4ae57; Hm_lvt_d8a99640d3ba3fdec41370651ce9b2ac=1752644342; Hm_lpvt_d8a99640d3ba3fdec41370651ce9b2ac=1752644342; HMACCOUNT=AE2B08950A2DD0B7; timestamp=1752644427208'
    text = HTTPTools.get(url='https://danjuanfunds.com/djapi/index_eva/pe_history/NDX?day=all', headers=headers)
    kvs = json.loads(text)['data']['index_eva_pe_growths']
    sql = 'insert into t_e(code,date,dwjz,ljjz) select ?,?,?,? where not exists(select 1 from t_e where code = ? and date = ?)'
    params = []
    for kv in kvs:
        code = 'NDX_PE'
        date = kv['ts'] / 1000
        date = time.localtime(date)
        date = time.strftime("%Y%m%d", date)
        dwjz = kv['pe']
        params.append((code, date, dwjz, dwjz, code, date))
    params.sort()
    rows = db.run(sql, params)
    print('NDX_PE', rows)


def fun2():
    Tools.echo('', 'E:/1.TXT', 'w')
    db = DBConn('sqlite', db='E:/MyFiles/MyPy/data/fund.db', poolsize=64)
    code = 'NDX'
    begin_date = '2000-09-26'
    end_date = '2025-05-29'
    
    money = 1
    share = 0
    hc = 0
    max_jz = 0
    cc = 0
    
    zf0 = 0
    zf1 = 0
    zf2 = 0
    
    rss = db.run('select date,val,zf from t_e where code = ? and date >= ? and date <= ?', (code, begin_date, end_date)).get_rows()
    
    for rs in rss:
        date = rs[0]
        jz = rs[1]
        zf = rs[2]
        
        zf2 = zf1
        zf1 = zf0
        zf0 = zf
        zflj = zf2 + zf1 + zf0
        
        max_jz = max(max_jz, jz)
        
        hc = (max_jz - jz) / max_jz * 100
        
        # cc = min(50 + int(hc / 10) * 10, 100)
        if zflj > 5:
            cc = 100
        elif zflj < -3:
            cc = 0
            
        if zflj > 3:
            cc = 100
        elif zflj < -5:
            cc = 0
            
        # cc = 100
        
        zje = money + share * jz
        
        share = zje * cc / 100 / jz
        money = zje * (100 - cc) / 100
        
        Tools.echo((date, round(jz, 2), round(zf, 2), round(cc, 2), round(share * jz + money, 2)), 'E:/1.TXT', 'a')
    

if __name__ == '__main__':
    # get_ndx_pe()
    # fun1()
    main()




# coding:utf-8
import sys
from tools_hjh import Log
from tools_hjh import ThreadPool
from tools_hjh import Tools
from bs4 import BeautifulSoup
from tools_hjh import HTTPTools
import os
from tools_hjh import ChromePool
from tools_hjh.Tools import str_exception, rm
import time

date = Tools.locatdate()
log = Log('E:/MyFiles/MyPy/log/sissy_game/' + date + '.log')

host = 'https://sissy.game'

chrome_path = r'D:\MyApps\CentBrowser\App\chrome.exe'
chromedriver_path = r'D:\MyApps\CentBrowser\chromedriver.exe'

headers = {
    'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
    'accept-language': 'zh-CN,zh;q=0.9',
    'cache-control': 'no-cache',
    'cookie':'66b513be114e386c60be7555c767e261latest_time_id=3794; theme_dark=0',
    'dnt': '1',
    'pragma': 'no-cache',
    'sec-ch-ua': '"Google Chrome";v="107", "Chromium";v="107", "Not=A?Brand";v="24"',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-platform': '"Windows"',
    'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36'
}

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = 'download'
try:
    sys_argv_2 = sys.argv[2]
except:
    sys_argv_2 = None
try:
    sys_argv_3 = sys.argv[3]
except:
    sys_argv_3 = 8
    

def main():
    req = ChromePool(1, chrome_path, chromedriver_path, is_display_picture=False, is_hidden=False)
    download('/page/PAGEIDX/', req)

    
def download(first_page_url, req):
    tp = ThreadPool(sys_argv_3)

    def download_one_page(idx, req):
        pageurl = host + first_page_url.replace('PAGEIDX', str(idx))
        page = req.get(pageurl)
        bs = BeautifulSoup(page, features="lxml")
        divs = bs.find_all('div', class_='single-post panel box-shadow-wrap-normal')
        for div in divs:
            name_ = div.find('h2', class_='m-t-none text-ellipsis index-post-title text-title').find('a').text
            date_ = div.find('div', class_='text-muted post-item-foot-icon text-ellipsis list-inline').find_all('li')[1].text
            date_ = date_.replace(' 年 ', '').replace(' 月 ', '').replace(' 日', '')
            try:
                i = 0
                e_url = div.find('div', class_='index-post-img').find('a')['href']
                e_page = req.get(e_url)
                bs = BeautifulSoup(e_page, features="lxml")
                for img in bs.find_all('img', class_='block'):
                    i = i + 1
                    pic_url = img['src']
                    if i > 1:
                        pic_name = date_ + '_' + name_ + '_' + str(i) + '.jpg'
                    else:
                        pic_name = date_ + '_' + name_ + '.jpg'
                    pic_name = Tools.merge_spaces(pic_name)
                    pic_name = pic_name.replace('/', ' ').replace('\\', ' ').replace(':', '：').replace('*', '·').replace('?', '？').replace('"', '\'').replace('<', '[').replace('>', ']').replace('|', ' ') 
                    pic_name = pic_name.replace('＃', '#')
                    pic_name = pic_name.replace('] ', ']').replace(' [', '[')
                    pic_name = pic_name.replace('～ ', '～').replace(' ～', '～')
                    pic_name = pic_name.replace('# ', '#').replace(' #', '#')
                    pic_name = pic_name.replace('。 ', '。').replace(' 。', '。')
                    pic_name = pic_name.replace('( ', '(').replace(' (', '(')
                    pic_name = pic_name.replace('！ ', '！').replace(' ！', '！')
                    pic_name = 'E:/MyFiles/MyPy/download/sissy_game/' + pic_name
                    if 'uploads' in pic_url:
                        tp.run(download_, (pic_url, pic_name, headers))
            except:
                log.error('download', idx, str(_))
    
    page = get_host(req, host)
    bs = BeautifulSoup(page, features="lxml")
    max_page = bs.find('ol', class_='page-navigator').find_all('li')[-2].text
    for idx in range(1, int(max_page) + 1):
        download_one_page(idx, req)
    req.close()
    tp.wait()

    
def download_(pic_url, pic_name, headers):
    try:
        if not os.path.exists(pic_name):
            HTTPTools.download(pic_url, pic_name, headers)
            log.info('download', 'no exists', pic_name, pic_url)
        else:
            net_size = HTTPTools.get_size(pic_url, headers)
            file_size = os.path.getsize(pic_name)
            if net_size == file_size:
                log.info('download', 'exists', pic_name, pic_url)
            else:
                rm(pic_name)
                HTTPTools.download(pic_url, pic_name, headers)
                log.info('download', 'no same size', pic_name, pic_url)
    except Exception as _:
        time.sleep(1)
        log.warning('download', 'retry', pic_name, pic_url)
        download_(pic_url, pic_name, headers)

        
def get_host(req, host):
    try:
        page = req.get(host)
    except:
        get_host(req, host)
    return page

    
if __name__ == '__main__':
    main()




# coding:utf-8
import sys
from tools_hjh import DBConn
from tools_hjh import Log
from tools_hjh import ThreadPool
from tools_hjh import Tools
from bs4 import BeautifulSoup
from tools_hjh.HTTPTools import HTTPTools

stop_flag = False
date = Tools.locatdate()
log = Log('E:/MyFiles/MyPy/log/fhxy/' + date + '.log')

host = 'https://fhxy-a.top'

chrome_path = r'D:\MyApps\CentBrowser\App\chrome.exe'
chromedriver_path = r'D:\MyApps\CentBrowser\chromedriver.exe'

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = 'download'
try:
    sys_argv_2 = sys.argv[2]
except:
    sys_argv_2 = None
try:
    sys_argv_3 = sys.argv[3]
except:
    sys_argv_3 = 32
    

def main(): 
    db = DBConn('sqlite', db='E:/MyFiles/MyPy/data/fhxy.db')
    createDatabase(db, rebuild=False)
    get_main(db, '/forum-2-PAGEIDX.html')
    get_main(db, '/forum-36-PAGEIDX.html')
    # get_main(db, '/forum-37-PAGEIDX.html')
    # get_main(db, '/forum-38-PAGEIDX.html')
    # get_main(db, '/forum-44-PAGEIDX.html')
    # get_main(db, '/forum-45-PAGEIDX.html')
    # get_main(db, '/forum-46-PAGEIDX.html')
    # get_main(db, '/forum-42-PAGEIDX.html')
    # get_e(db, HTTPTools)

    
def createDatabase(db, rebuild=False):
    if rebuild:
        db.run('drop table if exists t_m')
        db.run('drop table if exists t_e')

    t_m = '''
        create table if not exists t_m(
            url text, 
            name_ text,
            date text
        )
    '''
    
    t_e = '''
        create table if not exists t_e(
            url text, 
            name_ text, 
            content text
        )
    '''
    db.run(t_m)
    db.run(t_e)
    db.run('create index if not exists idx_m_url on t_m(url)')
    db.run('create index if not exists idx_e_url on t_e(url)')

    
def get_main(db, firstPageUrl):
    stop_flag = False

    def get_mian_one(idx, db, times=1):
        try:
            insert_params = []
            pageurl = host + firstPageUrl.replace('PAGEIDX', str(idx))
            page = HTTPTools.get(pageurl, timeout=10)
            bs = BeautifulSoup(page, features="lxml")
            objs = bs.find_all('div', class_='acgiflists')
            for obj in objs:
                try:
                    url = obj.find('a', class_='s xst')['href'].strip()
                    name_ = obj.find('a', class_='s xst').text.strip()
                    date_ = None
                    try:
                        date_ = obj.find('div', class_='acgifby1').find('span').find('span')['title']
                    except:
                        date_ = obj.find('div', class_='acgifby1').find('span').text
                    if date_ is not None:
                        date_1 = date_.split('-')[0]
                        date_2 = date_.split('-')[1]
                        if len(date_2) == 1:
                            date_2 = '0' + str(date_2)
                        date_3 = date_.split('-')[2]
                        if len(date_3) == 1:
                            date_3 = '0' + str(date_3)
                        date_ = date_1 + '-' + date_2 + '-' + date_3
                except Exception as _:
                    pass
                    # log.warning('get_mian_one', (url, name_, date_, url), str(_))
                
                insert_param = (url, name_, date_, url)
                insert_params.append(insert_param)
            
            insert_sql = 'insert into t_m select ?,?,? where not exists(select 1 from t_m t where t.url = ?)'
            insert_num = db.run(insert_sql, insert_params)
            log.info('get_mian_one', pageurl, insert_num, len(insert_params), times)
            
            if insert_num == 0:
                nonlocal stop_flag
                stop_flag = True
            
        except Exception as _:
            if times <= 99:
                times = times + 1 
                get_mian_one(idx, db, times)
            else:
                log.error('get_mian_one', pageurl, 'error', _)
    
    try:
        pageurl = host + firstPageUrl.replace('PAGEIDX', '1')
        page = HTTPTools.get(pageurl, timeout=10)
        bs = BeautifulSoup(page, features="lxml")
        page_num = int(bs.find('a', class_='last').text.replace('... ', ''))
    except Exception as _:
        log.error(_)
        get_main(db, firstPageUrl)

    tp = ThreadPool(sys_argv_3)
    for idx in range(1, page_num + 1):
        if stop_flag == False:
            tp.run_wait(get_mian_one, (idx, db))
        else:
            break
    tp.wait()

    
def get_e(db, chrome, tpNum=sys_argv_3):
    params = []

    def get_e_one(url, name_, times=1):
        try:
            page = chrome.get(url)
            bs = BeautifulSoup(page, features="lxml")
            content = Tools.merge_spaces(bs.find('div', class_='entry-content').text.strip()).strip()
            tags = ''
            tag_spans = bs.find_all('span', class_='tag-text')
            for span in tag_spans:
                tags = tags + span.text + '、'
            tags = tags.strip('、')
                
            param = (url, name_, content, tags, url)
            params.append(param)
            
            log.info('get_e_one', url, len(params), tp.get_running_num(), times)
        except Exception as _:
            if times <= 5:
                times = times + 1 
                get_e_one(url, name_, times)
            else:
                log.error('get_e_one', url, 'error', _)
    
    tp = ThreadPool(tpNum)
    urls = db.run('select url,name_ from t_m where not exists(select 1 from t_e where t_e.url = t_m.url and t_e.name_ = t_m.name_) order by date desc').get_rows()
    sql = 'insert into t_e select ?,?,?,? where not exists(select 1 from t_e t where t.url = ?)'
    for url in urls:
        tp.run(get_e_one, (url[0], url[1],), time_out=10)
        if len(params) > 500:
            tp.wait()
            num = db.run(sql, params)
            log.info('get_e', num)
            params.clear()
    tp.wait()
    num = db.run(sql, params)
    log.info('get_e', num)


def sql(db):
    sql = '''
        select t.url,t.date,t.views
        from t_m t left join t_e e
        on t.url = e.url 
        where 1=1
        and e.content not like '%GPT%' and e.content not like '%AI%' and e.content not like '%机翻%' and e.content not like '%智能翻译%' and e.content not like '%云汉%' and e.content not like '%云翻%' and e.content not like '%生肉%' and e.content not like '%智能汉化%'
        and t.name_ not like '%GPT%' and t.name_ not like '%AI%' and t.name_ not like '%机翻%' and t.name_ not like '%智能翻译%' and t.name_ not like '%云汉%' and t.name_ not like '%云翻%' and t.name_ not like '%生肉%' and t.name_ not like '%智能汉化%'
        --and e.tags not like '%ADV%'
        --and e.content not like '%欧美%'
        and (
            e.tags like '%战斗H%' or e.tags like '%战斗h%' or e.tags like '%戰鬥H%' or e.tags like '%戰鬥h%' or e.tags like '%战斗エロ%' or e.tags like '%戰鬥エロ%'
            or e.content like '%战斗H%' or e.content like '%战斗h%' or e.content like '%戰鬥H%' or e.content like '%戰鬥h%' or e.content like '%战斗エロ%' or e.content like '%戰鬥エロ%'
        ) order by t.views desc;
    '''
    rss = db.run(sql).get_rows()
    for rs in rss:
        print(rs)

    
if __name__ == '__main__':
    main()




# coding:utf-8
import sys
from tools_hjh import Log
from tools_hjh import ThreadPool
from tools_hjh import Tools
from bs4 import BeautifulSoup
from tools_hjh import ChromePool
import json
from tools_hjh import HTTPTools
from tools_hjh.Tools import echo, rm, cat
import ast

Tools.unzip_chromedriver()

Tools.mkdir('log')
Tools.mkdir('tmp')
Tools.mkdir('data/download')

date = Tools.locatdate()
log = Log('log/' + date + '.log')

host = 'https://avbebe.com/'

chrome_path = r'D:\MyApps\CentBrowser\App\chrome.exe'
chromedriver_path = r'D:\MyApps\CentBrowser\chromedriver.exe'

headers = {
    'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
    'accept-language': 'zh-CN,zh;q=0.9',
    'cache-control': 'no-cache',
    'dnt': '1',
    'pragma': 'no-cache',
    'sec-ch-ua': '".Not/A)Brand";v="99", "Google Chrome";v="103", "Chromium";v="103"',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-platform': '"Windows"',
    'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36',
    'Referer': 'https://avbebe.com/'
}

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = 'download'
try:
    sys_argv_2 = sys.argv[2]
except:
    sys_argv_2 = None
try:
    sys_argv_3 = sys.argv[3]
except:
    sys_argv_3 = 8
    
# SELECT 'm_pages.append(('''||url||''', '''||name||'【'||date||'】.mp4'||'''))' FROM T_M t where t.date like '2023%' order by date;'


def main(): 
    m_pages = []
    m_pages.append(('https://avbebe.com/archives/137431', 'けものっ娘通信〜The Animation〜 猫娘ニア[中文字幕]【20230102】.mp4'))
    
    params = get_url(m_pages, sys_argv_3)
    echo(str(params), 'tmp/avbebe_lifan.par')

    download('tmp/avbebe_lifan.par', r'E:\avbebe')
    # rm('tmp/avbebe_lifan.par')

    
def get_url(m_pages, sys_argv_3):
    params = []
    chrome = ChromePool(sys_argv_3, chrome_path, chromedriver_path, is_display_picture=False, is_hidden=False)

    def get_one(m_page, name, chrome):
        page = chrome.get(m_page)
        bs = BeautifulSoup(page, features="lxml")
        
        # type
        try:
            as_ = bs.find('div', class_='jeg_meta_category').find_all('tr')
            type_ = ''
            for a in as_:
                typ_ = a.text
                type_ = type_ + typ_ + '、'
            type_ = type_[0:-1]
        except Exception as _:
            log.error('getE', m_page, 'tag not find')
            type_ = None
        
        # tags
        try:
            tags = ''
            as_ = bs.find('div', class_='jeg_post_tags').find_all('a', rel='tag')
            for a in as_:
                tag = a.text
                tags = tags + tag + '、'
            tags = tags[0:-1]
        except:
            tags = None
        
        # name_ video_url picture_url subtitles_url
        if '.mp4' not in page.lower() and '.m3u8' not in page.lower():
            Tools.echo(m_page, 'no_video.txt', 'a')
            log.error('getE', m_page, '不存在链接')
        else: 
            try:  # 一类json
                content = bs.find('div', attrs={"id":True, "data-item":True})['data-item']
                obj = json.loads(content)
                try:
                    if name is None:
                        name_ = obj['fv_title']
                    else:
                        name_ = name
                except:
                    name_ = ''
                    
                try:
                    video_url = ''
                    sources = obj['sources']
                    for source in sources:
                        video_url = video_url + source['src'] + '、'
                    video_url = video_url[0:-1]
                except:
                    video_url = None
                    
                try:
                    picture_url = obj['splash']
                except:
                    picture_url = None
                    
                try:
                    subtitles_url = ''
                    subtitles = obj['subtitles']
                    for subtitle in subtitles:
                        subtitles_url = subtitles_url + subtitle['src'] + '、'
                    subtitles_url = subtitles_url[0:-1]
                except:
                    subtitles_url = None
                    
                if video_url is not None:
                    param = (m_page, name_, picture_url, video_url, subtitles_url, tags, type_)
                    params.append(param)
            except:
                content = None
            
            if content is None:  # 二类json
                try:
                    as_ = bs.find_all('a', attrs={"data-fancybox":True, "itemprop":True, "data-item":True, "itemscope":True, "itemtype":True, "href":True})
                    for a in as_:
                        content = a['data-item']
                        obj = json.loads(content)
                        try:
                            if name is None:
                                name_ = obj['fv_title']
                            else:
                                name_ = name
                        except:
                            name_ = ''
                        try:
                            video_url = ''
                            sources = obj['sources']
                            for source in sources:
                                video_url = video_url + source['src'] + '、'
                            video_url = video_url[0:-1]
                        except:
                            video_url = None
                        try:
                            picture_url = obj['splash']
                        except:
                            picture_url = None
                        try:
                            subtitles_url = ''
                            subtitles = obj['subtitles']
                            for subtitle in subtitles:
                                subtitles_url = subtitles_url + subtitle['src'] + '、'
                            subtitles_url = subtitles_url[0:-1]
                        except:
                            subtitles_url = None
                            
                        if video_url is not None:
                            param = (m_page, name_, picture_url, video_url, subtitles_url, tags, type_)
                            params.append(param)
                except:
                    content = None
                    
    if sys_argv_3 < 4:
        sys_argv_3 = 4
    tp = ThreadPool(sys_argv_3)
    for m_page, name in m_pages:
        tp.run(get_one, (m_page, name, chrome))
    tp.wait()
                
    chrome.close()
    return params


def download(params, filepath_=r'E:/avbebe_lifan'):
    headers_ = 'referer:https://avbebe.com/'
    exe_path = 'E:/MyApps/N_m3u8DL-CLI/N_m3u8DL-CLI_v3.0.2.exe'

    def d_one(param):
        try:
            video_url = param[3]
            subtitles_url = param[4]
            name = param[1].replace('/', ' ').replace('\\', ' ').replace(':', '：').replace('*', '·').replace('?', '？').replace('"', '\'').replace('<', '[').replace('>', ']').replace('|', ' ')
            name = Tools.merge_spaces(name)
            name = name.replace('＃', '#')
            name = name.replace('] ', ']').replace(' [', '[')
            name = name.replace('～ ', '～').replace(' ～', '～')
            name = name.replace('# ', '#').replace(' #', '#')
            name = name.replace('。 ', '。').replace(' 。', '。')
            name = name.replace('( ', '(').replace(' (', '(')
            name = name.replace('！ ', '！').replace(' ！', '！')
            name = name.replace('OVA ', 'OVA').replace(' OVA', 'OVA')
            if subtitles_url != 'None' and subtitles_url is not None:
                subtitles_flags = False
                if '、' in subtitles_url:
                    for s_url in subtitles_url.split('、'):
                        if '簡' in s_url or '简' in s_url:
                            HTTPTools.download(s_url, filepath=filepath_ + '/' + name + '.vtt', headers=headers)
                            subtitles_flags = True
                    if not subtitles_flags:
                        HTTPTools.download(subtitles_url[0], filepath=filepath_ + '/' + name + '.vtt', headers=headers)
                else:
                    HTTPTools.download(subtitles_url, filepath=filepath_ + '/' + name + '.vtt', headers=headers)                    
            if video_url.endswith('.m3u8'):
                HTTPTools.N_m3u8DL_CLI(url=video_url, filepath=filepath_, filename=name, theadnum=32, exe_path=exe_path, headers=headers_)
            elif video_url.endswith('.mp4'):
                HTTPTools.download(video_url, filepath=filepath_ + '/' + name + '.mp4', headers=headers)
            echo(str(param), 'tmp/downloaded.par')
        except Exception as _:
            echo(str(param), 'tmp/exception.par')
        
    params = cat(params)
    params = ast.literal_eval(params)
    tp = ThreadPool(4)
    for param in params:
        if str(param) not in cat('tmp/downloaded.par') and str(param) not in cat('tmp/exception.par'):
            tp.run(d_one, (param,))
    tp.wait()


if __name__ == '__main__':
    main()
    



from tools_hjh.Chrome import Chrome
from selenium.webdriver.common.by import By
from bs4 import BeautifulSoup
from tools_hjh import HTTPTools, Tools, Log
import json
import re
import os
import sys

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = ''
    
headers = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
    "Referer": "https://www.bilibili.com/",
    "cookie":"buvid3=77CE9384-58A9-6FA8-2887-43BDFB8FB13153775infoc; b_nut=1738808053; _uuid=5AFD7B76-F3D5-9556-81044-310CEEB28FFBA54661infoc; buvid_fp=a707572b4a023295611d25377e5200ba; enable_web_push=DISABLE; home_feed_column=5; buvid4=B9CC925B-E9EC-FCF6-521D-CDB4DB42924F57462-025020602-nod6mU0ldP+etlR5l7OO+w==; DedeUserID=84131551; DedeUserID__ckMd5=15aa2aee95cb2c8f; hit-dyn-v2=1; header_theme_version=CLOSE; rpdid=|(JJml)mJJ~)0J'u~Jmklu~~J; enable_feed_channel=ENABLE; LIVE_BUVID=AUTO3917413704698685; PVID=4; bsource=search_bing; bmg_af_switch=1; bmg_src_def_domain=i1.hdslb.com; bili_ticket=eyJhbGciOiJIUzI1NiIsImtpZCI6InMwMyIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NDM0MjAyNjQsImlhdCI6MTc0MzE2MTAwNCwicGx0IjotMX0.hR5CWBt08cSgh5kpRlXn-lktrgPEOprhIU1mfcuY7Is; bili_ticket_expires=1743420204; SESSDATA=f46dd477,1758713389,a02d4*32CjBFIxZez9Yw4EsHKfFeEW3KUxe8ntUHbCx-84Bm5AMojY1_LZtsPCUonLkXDN8Ndf0SVkpfMmJTUGZGbXdtcG5vd29CNWhqRUZ5Y1FwckxkTW95MXlXQjl3clZkRExiNWNVb3FZU21vVXAtdkllNHBjRlhkbkpma3VBVWxCanJPdW1kWUZEellRIIEC; bili_jct=b369a500c36834d6d00fc406677d9de9; sid=7sivgz0o; b_lsid=DFCEEA9E_195DCD22A42; CURRENT_QUALITY=32; bp_t_offset_84131551=1049386541933133824; CURRENT_FNVAL=2000; browser_resolution=1536-190"
}

date = Tools.locatdate()
log = Log(date + '.log')


def main():
    chrome_path = r'D:\MyApps\CentBrowser\App\chrome.exe'
    chromedriver_path = r'D:\MyApps\CentBrowser\chromedriver.exe'
    
    cp = Chrome(chrome_path=chrome_path, chromedriver_path=chromedriver_path, is_hidden=False, is_display_picture=False, headers=headers)
    page = cp.get('https://www.bilibili.com/video/' + sys_argv_1)
    volumes = cp.chrome.find_elements(By.CLASS_NAME, 'slide-item')
    
    all_list = []
    for volume in volumes:
        try:
            volume.click()
        except:
            continue
        page = cp.chrome.page_source
        bs = BeautifulSoup(page, features="lxml")
        videos = bs.find('div', class_='video-pod__body').find_all('div', class_='pod-item video-pod__item simple')
        for video in videos:
            url = 'https://www.bilibili.com/video/' + video['data-key'] + '?t=16.9'
            title = video.find('div', class_='title-txt').text.replace(' ', '_')
            all_list.append((url, title))
    cp.close()

    i = 1
    for one in all_list:
        if i < 9:
            tt = '0' + str(i)
        else:
            tt = str(i)
        url = one[0]
        title = tt + '_' + one[1]
        v_page = HTTPTools.get(url, headers=headers)
        script_content = re.search(r'<script>window\.__playinfo__=(.*?)</script>', v_page).group(1)
        Tools.echo(script_content, 'd:/1.txt', 'w')
        video_info = json.loads(script_content)
        video_url = ''
        if len(video_info['data']['dash']['video']) == 1:
            video_url = video_info['data']['dash']['video'][0]['baseUrl']
        for v in video_info['data']['dash']['video']:
            if v['height'] == 1080:
                continue
            if v['height'] == 720:
                video_url = v['baseUrl']
                break
            if v['height'] == 480 and video_url == '':
                video_url = v['baseUrl']
                break
            if v['height'] == 360 and video_url == '':
                video_url = v['baseUrl']
                break
        if video_url == '':
            video_url = video_info['data']['dash']['video'][0]['baseUrl']
        audio_url = video_info['data']['dash']['audio'][0]['baseUrl']
        
        mp4_size_1 = HTTPTools.get_size(video_url, headers=headers)
        mp3_size_1 = HTTPTools.get_size(audio_url, headers=headers)
        
        mp4 = title + '_a.mp4'
        mp3 = title + '_b.mp4'
        
        if os.path.exists(mp4):
            mp4_size_2 = os.path.getsize(mp4)
        else:
            mp4_size_2 = 0
            
        if os.path.exists(mp3):
            mp3_size_2 = os.path.getsize(mp3)
        else:
            mp3_size_2 = 0
        
        if mp4_size_1 > mp4_size_2:
            log.info(mp4, 'begin', 'src=' + str(round(mp4_size_1 / 1024 / 1024, 2)) + 'MB', 'dst=' + str(round(mp4_size_2 / 1024 / 1024, 2)) + 'MB')
            Tools.rm(mp4)
            try:
                HTTPTools.download(video_url, mp4, headers)
                log.info(mp4, 'begin', 'src=' + str(round(mp4_size_1 / 1024 / 1024, 2)) + 'MB', 'dst=' + str(round(os.path.getsize(mp4) / 1024 / 1024, 2)) + 'MB')
            except:
                Tools.rm(mp4)
        else:
            log.warning(mp4 + ' exists')
        
        if mp3_size_1 > mp3_size_2:
            log.info(mp3, 'begin', 'src=' + str(round(mp3_size_1 / 1024 / 1024, 2)) + 'MB', 'dst=' + str(round(mp3_size_2 / 1024 / 1024, 2)) + 'MB')
            Tools.rm(mp3)
            try:
                HTTPTools.download(audio_url, mp3, headers)
                log.info(mp3, 'begin', 'src=' + str(round(mp3_size_1 / 1024 / 1024, 2)) + 'MB', 'dst=' + str(round(os.path.getsize(mp3) / 1024 / 1024, 2)) + 'MB')
            except:
                Tools.rm(mp3)
        else:
            log.warning(mp3 + ' exists')
            
        i = i + 1
    

if __name__ == '__main__':
    main()



# coding:utf-8
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
import time
from tools_hjh.ThreadPool import ThreadPool
import os
from tools_hjh import Tools
from tools_hjh.Tools import mkdir, rm


def main():
    pass


class Chrome():
    """ 使用浏览器解析url，返回源码
        __init__.param：
            chrome_path: chrome.exe路径
            chromedriver_path: chromedriver.exe路径
    """

    def __init__(self, chrome_path, chromedriver_path, is_hidden=False, is_display_picture=True, headers=None):
        chrome_options = Options()
        # chrome_options.add_argument("--single-process")
        # chrome_options.add_argument("--disable-site-isolation-trials")

        if is_hidden:
            chrome_options.add_argument("--headless")
        chrome_options.binary_location = chrome_path
        if not is_display_picture:
            chrome_options.add_experimental_option('prefs', {'profile.managed_default_content_settings.images': 2})
        if headers is not None:
            for key, value in headers.items():
                header = key + "=" + value
                chrome_options.add_argument(header)
        self.chrome = webdriver.Chrome(chromedriver_path, options=chrome_options)
        self.status = 0  # 未被占用
    
    def set_cookies(self, cookies, cookie_domian):
        self.get(cookie_domian)
        for cookie in cookies:
            self.chrome.add_cookie(cookie)
        
    def close(self):
        self.chrome.quit()
        
    def get(self, url):
        self.status = 1  # 被占用
        self.chrome.get(url)
        text = self.chrome.page_source
        self.status = 0
        return text
    
    def download(self, url, filepath, replace=False):
        """ 下载请求的文件, 返回文件大小 """
        filepath = filepath.replace('\\', '/')
        
        if '/' in filepath:
            path = filepath.rsplit('/', 1)[0] + '/'
            mkdir(path)
            
        if os.path.exists(filepath) and replace:
            rm(filepath)
        elif os.path.exists(filepath) and not replace:
            return 0

        try:
            self.get(url)
            screenshot_data = self.chrome.get_screenshot_as_png()  # 获取二进制数据
            with open(filepath, "wb") as f:
                f.write(screenshot_data)
        except Exception as _:
            pass
        finally:
            try:
                f.close()
            except:
                pass
        
        download_size = os.path.getsize(filepath)

        return download_size
    
    def get_status(self):
        return self.status


class ChromePool():

    def __init__(self, maxSize, chrome_path, chromedriver_path, is_hidden=False, is_display_picture=True, cache=False, headers=None):
        self.cache = cache
        self.maxSize = maxSize
        self.pool = []
        for _ in range(0, maxSize):
            self.pool.append(Chrome(chrome_path, chromedriver_path, is_hidden, is_display_picture, headers))
        self.openSize = 0
        
    def set_cookies(self, cookies, cookie_domian):

        def set_one(chrome, cookies, cookie_domian):
            chrome.set_cookies(cookies, cookie_domian)

        tp = ThreadPool(self.maxSize)
        for chrome in self.pool:
            tp.run(set_one, (chrome, cookies, cookie_domian))
        tp.wait()
        
    def get(self, url):
        while self.openSize == self.maxSize:
            time.sleep(0.2)
        for chrome in self.pool:
            if chrome.get_status() == 0:
                self.openSize = self.openSize + 1
                if self.cache:
                    url_split = url.split('/')
                    host = url_split[0] + '//' + url_split[2] + '/'
                    path_ = 'tmp/' + url.replace(host, '').rstrip('/')
                    if os.path.exists(path_):
                        text = Tools.cat(path_)
                    else:
                        text = chrome.get(url)
                        Tools.echo(text, path_)
                else:
                    text = chrome.get(url)
                self.openSize = self.openSize - 1
                return text
        time.sleep(0.2)
        return self.get(url)

    def download(self, url, filepath, replace=False):
        while self.openSize == self.maxSize:
            time.sleep(0.2)
        for chrome in self.pool:
            if chrome.get_status() == 0:
                self.openSize = self.openSize + 1
                size = chrome.download(url, filepath, replace)
                self.openSize = self.openSize - 1
                return size
        time.sleep(0.2)
        return self.download(url, filepath, replace)
    
    def close(self):
        while self.openSize > 0:
            time.sleep(0.2)
        for c in self.pool:
            c.close()
            

if __name__ == '__main__':
    main()
